#!/usr/bin/perl

####################################################################
#  Perl interface to CommuniGate Pro CLI.
#
#  Version 2.6.5
#
#  Original location: <http://www.stalker.com/CGPerl>
#  Revision history: <http://www.stalker.com/CGPerl/History.html>
#
#  See <http://www.stalker.com/CommuniGatePro/CLI.html> for the related info.
#
#  Mail your comments and error reports to <support@stalker.com>

############################## Commands:
# new
# Logout
# getErrCode
# getErrMessage
# isSuccess
# getErrCommand
# setStringsTranslateMode
# NewPassword
# SendCommand
# GetResponseData

############################## Accounts commands
# ListAccounts
# CreateAccount
# RenameAccount
# DeleteAccount
# GetAccountSettings
# GetAccountEffectiveSettings
# UpdateAccountSettings
# SetAccountSettings
# SetAccountPassword
# VerifyAccountPassword
# GetAccountAliases
# SetAccountAliases
# GetAccountRules
# SetAccountRules
# GetAccountRPOP
# SetAccountRPOP
# GetAccountRights
# SetAccountRights
# GetAccountInfo
# GetWebUser
# SetWebUser
# GetEffectiveWebUser

############################## Group Commands
# ListGroups
# CreateGroup
# RenameGroup
# DeleteGroup
# GetGroup
# SetGroup

############################## Forwarder Commands
# ListForwarders
# CreateForwarder
# DeleteForwarder
# GetForwarder

############################## Domain commands
# ListDomains
# MainDomainName
# [Get|Update|Set]DomainSettings
# GetDomainEffectiveSettings
# [Create|Rename|Delete]Domain
# CreateSharedDomain
# CreateDirectoryDomain
# [Get|Set]DomainRules
# [Get|Set]DomainAliases
# ListAdminDomains
# [Insert|Delete]DirectoryRecords
# [Get|Set]DirectoryIntegration
# [Get|Set]ClusterDirectoryIntegration


# [Get|Update|Set]DomainDefaults
# [Get|Update|Set]ClusterDomainDefaults
# [Get|Update|Set]AllAccountsDefaults
# GetDomainLocation
# GetAccountLocation
# [Get|Update|Set]AccountDefaults
# [Get|Update|Set]ClusterAccountDefaults
# [Get|Set]WebUserDefaults
# [Get|Set]ServerWebUserDefaults
# [Get|Set]ClusterWebUserDefaults
# [Get|Update|Set]AccountTemplate

############################## Mailbox Administration
# ListMailboxes
# [Create|Rename|Delete]Mailbox
# [Rename|Delete]Mailboxes
# GetMailboxInfo
# [Get|Set]MailboxACL
# GetMailboxRights
# SetMailboxClass

# [Get|Set]AccountSubscription
# [Get|Set]MailboxAliases

############################## Alerts Administration
# Get[Domain|Account|Server|Cluster]Alerts
# Set[Domain|Account|Server|Cluster]Alerts
# Post[Domain|Account|Server|Cluster]Alert
# Remove[Domain|Account|Server|Cluster]Alert

############################## Personal Web Site Administration
# GetWebFile
# PutWebFile
# RenameWebFile
# DeleteWebFile
# ListWebFiles
# GetWebFilesInfo

############################## Lists commands
# ListLists
# GetDomainLists
# GetAccountLists
# [Create|Rename|Delete|Get|Update]List
# List
# ListSubscribers
# GetSubscriberInfo
# SetPostingMode
# ProcessBounce

############################## Web Skins Administration
# ListDomainSkins( [domainName] )
# CreateDomainSkin(domainName,skinName)
# RenameDomainSkin(domainName,oldSkinName,newSkinName)
# DeleteDomainSkin(domainName,skinName)
# ListDomainSkinFiles(domainName,skinName)
# ReadDomainSkinFile(domainName,skinName,fileName)
# StoreDomainSkinFile(domainName,skinName,fileName,base64data)
# DeleteDomainSkinFile(domainName,skinName,fileName)

# ListServerSkins()
# CreateServerSkin(skinName)
# RenameServerSkin(oldSkinName,newSkinName)
# DeleteServerSkin(skinName)
# ListServerSkinFiles(skinName)
# ReadServerSkinFile(skinName,fileName)
# StoreServerSkinFile(skinName,fileName,base64data)
# DeleteServerSkinFile(skinName,fileName)

# ListClusterSkins()
# CreateClusterSkin(skinName)
# RenameClusterSkin(oldSkinName,newSkinName)
# DeleteClusterSkin(skinName)
# ListClusterSkinFiles(skinName)
# ReadClusterSkinFile(skinName,fileName)
# StoreClusterSkinFile(skinName,fileName,base64data)
# DeleteClusterSkinFile(skinName,fileName)

############################## Web Interface Tuning
# ListWebUserInterface
# GetWebUserInterface
# PutWebUserInterface
# DeleteWebUserInterface
# ClearWebUserCache

############################## Web Interface Integration
# [Create|Get|Kill]WebUserSession

############################## Server commands
# [Get|Update|Set]Module

# [Get|Set]LANIPs
# [Get|Set]BlacklistedIPs
# [Get|Set]ClientIPs
# [Get|Set]WhiteHoleIPs
# [Get|Set]Protection
# [Get|Set]Banned

# [Get|Set]ClusterLANIPs
# [Get|Set]ClusterBlacklistedIPs
# [Get|Set]ClusterClientIPs
# [Get|Set]ClusterWhiteHoleIPs
# [Get|Set]ClusterProtection
# GetClusterBanned

# [Get|Set][Server|Cluster]Rules
# RefreshOSData
# [Get|Set]RouterTable
# [Get|Set]RouterSettings
# [Get|Set]ClusterRouterTable
# [Get|Set][Server|Cluster]Intercept

# Route

############################## Monitoring commands
# GetSNMPElement
# Shutdown

############################## Statistics commands
# [Get|Reset]AccountStat
# [Get|Reset]DomainStat

############################## Miscellaneous commands
# WriteLog
# ReleaseSMTPQueue
# RejectQueueMessage
# GetCurrentController
# GetTempClientIPs
# RemoveAccountSubset

##############################################################

package CGP::CLI;

use strict;
use Carp;

# You may need to change this to "use IO::Socket::INET;" if you have INET.pm
use IO::Socket;

# You may need to install this module.
# See <http://search.cpan.org/search?module=Digest::MD5>
use Digest::MD5;


@CGP::CLI::ISA = qw(IO::Socket::INET);

$CGP::FALSE = 0;
$CGP::TRUE = 1;

$CGP::ERR_STRING = "No error";

$CGP::CLI_CODE_OK = 200;
$CGP::CLI_CODE_OK_INLINE = 201;

$CGP::CLI_CODE_PASSWORD = 300;
$CGP::CLI_CODE_UNKNOW_USER = 500;
$CGP::CLI_CODE_GEN_ERR = 501;
$CGP::CLI_CODE_STRANGE = 10000;

$CGP::CLI_CRLF = "\012";

$CGP::SECURE_LOGIN = 1;
$CGP::WEBUSER_LOGIN = 0;


sub new {
  my ($class, $params) = @_;

  my $login = delete $params->{'login'};
  my $password = delete $params->{'password'};

  my $isSecureLogin = delete $params->{'SecureLogin'};
  my $isWebUserLogin = delete $params->{'WebUserLogin'};


  $isSecureLogin = $CGP::SECURE_LOGIN unless defined $isSecureLogin;
  $isWebUserLogin = $CGP::WEBUSER_LOGIN unless defined $isWebUserLogin;


  croak 'You must pass login parameter to CGP::CLI::new'
    unless defined $login;
  croak 'You must pass password parameter to CGP::CLI::new'
    unless defined $password;

  croak 'SecureLogin and WebUserLogin are mutually exclusive'
    if($isSecureLogin && $isWebUserLogin);

  #print %$params;

  my $this = SUPER::new $class (%$params);

  unless(defined $this && $this) {
    $CGP::ERR_STRING="Can't open connection to CGPro Server";
    return undef;
  };

  $this->autoflush(1);

  unless($this->_parseResponse()) {
    $CGP::ERR_STRING="Can't read CGPro Server prompt";
    return undef;
  };

  if($isSecureLogin) {
    ${*$this}{'errMsg'} =~ /(\<.*\@*\>)/;

    my $md5=Digest::MD5->new;
    if($md5) {
      $md5->add($1.$password);
      $this->send('APOP '.$login.' '.$md5->hexdigest);
      $this->_parseResponse();
    } else {
      $CGP::ERR_STRING="Can't create MD5 object";
      close($this);
      return undef;
    }
  } elsif($isWebUserLogin) {
    $this->send('AUTH WEBUSER '.$login.' '.$password);
    $this->_parseResponse();

  } else {
    $this->send('USER '.$login);
    $this->_parseResponse();
    return $CGP::FALSE unless ${*$this}{'errCode'} == $CGP::CLI_CODE_PASSWORD;
    $this->send('PASS '.$password);
    $this->_parseResponse();
  }

  unless($this->isSuccess) {
    $CGP::ERR_STRING=${*$this}{'errMsg'};
    close($this);
    return undef;
  }
  $this->send('INLINE');
  $this->_parseResponse();
  $this->setStringsTranslateMode(0);

  $this;
}

sub getErrCode {
  my $this = shift;
  return ${*$this}{'errCode'};
}

sub getErrMessage {
  my $this = shift;
  return ${*$this}{'errMsg'};
}

sub getErrCommand {
  my $this = shift;
  return ${*$this}{'currentCGateCommand'};
}

sub isSuccess {
  my $this = shift;
  return (${*$this}{'errCode'} == $CGP::CLI_CODE_OK || ${*$this}{'errCode'} == $CGP::CLI_CODE_OK_INLINE);
}

sub setDebug {
  my ($this, $debugFlag) = @_;
  ${*$this}{'debug'} = $debugFlag;
}

sub setStringsTranslateMode {
  my ($this, $onFlag) = @_;
  ${*$this}{'translateStrings'} = $onFlag;
}

sub Logout {
  my $this = shift;
  $this->send('QUIT');
  $this->_parseResponse();
}

sub NewPassword {
  my ($this, $newPassword) = @_;
  croak 'usage CGP::CLI->NewPassword($newPassword)'
    unless defined $newPassword;
  $this->send('NEWPASS '.$newPassword);
  $this->_parseResponse();
}

sub SendCommand {
  my ($this, $command) = @_;
  croak 'usage CGP::CLI->SendCommand($commandString)'
    unless defined $command;
  $this->send($command);
  $this->_parseResponse();
}

sub GetResponseData {
  my ($this) = @_;
  $this->parseWords($this->getWords);
}


####################################################################
#    Account commands

sub ListAccounts {
  my ($this, $domainName) = @_;
  my $line = 'ListAccounts';
  $line .= ' '.$domainName if $domainName;
  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub CreateAccount {
  my ($this) = shift;;
  my %params = (@_);

  croak 'usage CGP::CLI->CreateAccount(accountName => \'newAccountName\',
                            accountType => \'accountType\',
                            externalFlag => \'externalFlag\',
                            settings     => $settings)'
      unless exists $params{accountName};
  my $line = 'CreateAccount '.$params{accountName};
  $line .= ' ' . $params{accountType} if exists $params{accountType};
  $line .= ' ' . 'external' if exists $params{externalFlag};
  $line .= ' ' . $this->printWords($params{settings}) if exists $params{settings};

  $this->send($line);
  $this->_parseResponse();
}

sub RenameAccount {
  my ($this, $oldAccountName, $newAccountName) = @_;
  croak 'usage CGP::CLI->RenameAccount($oldAccountName, $newAccountName)'
      unless defined $oldAccountName && defined $newAccountName;
  $this->send('RenameAccount '.$oldAccountName.' into '.$newAccountName);
  $this->_parseResponse();
}

sub DeleteAccount {
  my ($this, $accountName) = @_;
  croak 'usage CGP::CLI->DeleteAccount($accountName)'
    unless defined $accountName;
  $this->send('DeleteAccount '.$accountName);
  $this->_parseResponse();
}

sub GetAccountSettings {
  my ($this, $accountName) = @_;
  croak 'usage CGP::CLI->GetAccountSettings($accountName)'
    unless defined $accountName;
  $this->send('GetAccountSettings '.$accountName);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetAccount {
  GetAccountSettings(@_);
}

sub GetAccountEffectiveSettings {
  my ($this, $accountName) = @_;
  croak 'usage CGP::CLI->GetAccountEffectiveSettings($accountName)'
    unless defined $accountName;
  $this->send('GetAccountEffectiveSettings '.$accountName);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub UpdateAccountSettings {
  my ($this, $accountName, $params) = @_;
  croak 'usage CGP::CLI->UpdateAccountSettings($accountName, \%settings)'
    unless defined $accountName;
  $this->send('UpdateAccountSettings '.$accountName.' '.$this->printWords($params));
  $this->_parseResponse();
}

sub UpdateAccount {
  UpdateAccountSettings(@_);
}

sub SetAccountSettings {
  my ($this, $accountName, $params) = @_;
  croak 'usage CGP::CLI->SetAccountSettings($accountName, \%settings)'
    unless defined $accountName;
  $this->send('SetAccountSettings '.$accountName.' '.$this->printWords($params));
  $this->_parseResponse();
}

sub SetAccount {
  SetAccountSettings(@_);
}

sub SetAccountPassword {
  my ($this, $accountName, $newPass) = @_;
  croak 'usage CGP::CLI->SetAccountPassword($accountName, $newPassword)'
    unless defined $accountName && defined $newPass;
  $this->send('SetAccountPassword '.$accountName.' TO '.$this->printWords($newPass));
  $this->_parseResponse();
}

sub VerifyAccountPassword {
  my ($this, $accountName, $pass) = @_;
  croak 'usage CGP::CLI->VerifyAccountPassword($accountName, $password)'
    unless defined $accountName && defined $pass;
  $this->send('VerifyAccountPassword '.$accountName.' PASSWORD '.$this->printWords($pass));
  $this->_parseResponse();
}


sub GetAccountAliases {
  my ($this, $accountName) = @_;
  croak 'usage CGP::CLI->GetAccountAliases($accountName)'
    unless defined $accountName;
  $this->send('GetAccountAliases '.$accountName);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetAccountAliases {
  my ($this, $accountName, $aliases) = @_;
  croak 'usage CGP::CLI->SetAccountAliases($accountName, \@aliases)'
    unless defined $accountName && defined $aliases;
  $this->send('SetAccountAliases '.$accountName.' '.$this->printWords($aliases));
  $this->_parseResponse();
}

sub GetAccountRules {
  my ($this, $accountName) = @_;
  croak 'usage CGP::CLI->GetAccountRules($accountName)'
    unless defined $accountName;
  $this->send('GetAccountRules '.$accountName);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetAccountRules {
  my ($this, $accountName, $rules) = @_;
  croak 'usage CGP::CLI->SetAccountRules($accountName, \@rules)'
    unless defined $accountName && defined $rules;
  $this->send('SetAccountRules '.$accountName.' '.$this->printWords($rules));
  $this->_parseResponse();
}

sub GetAccountRPOP {
  my ($this, $accountName) = @_;
  croak 'usage CGP::CLI->GetAccountRPOP($accountName)'
      unless defined $accountName;
  $this->send('GetAccountRPOP '.$accountName);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetAccountRPOP {
  my ($this, $accountName, $newRecords) = @_;
  croak 'usage CGP::CLI->SetAccountAliases($accountName, \@newRecords)'
    unless defined $accountName && defined $newRecords;
  $this->send('SetAccountRPOP '.$accountName.' '.$this->printWords($newRecords));
  $this->_parseResponse();
}

sub GetAccountRights {
  my ($this, $account) = @_;
  croak 'usage CGP::CLI->GetAccountRights($account)'
    unless defined $account;
  $this->send('GetAccountRights '.$account);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetAccountRights {
  my ($this, $account, $list) = @_;
  croak 'usage CGP::CLI->SetAccountRigthts($acount, \@list)'
    unless defined $account && defined $list;
  my $line = 'SetAccountRights '.$account.' '.$this->printWords($list);

  $this->send($line);
  $this->_parseResponse();
}

sub GetAccountInfo {
  my ($this, $account, $key) = @_;
  croak 'usage CGP::CLI->GetAccountInfo($account,$key)'
    unless defined $account && defined $key;
  $this->send('GetAccountInfo '.$account.' Key '.$key);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetWebUser {
  my ($this, $account) = @_;
  croak 'usage CGP::CLI->GetWebUser($account)'
    unless defined $account;
  $this->send('GetWebUser '.$account);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetWebUser {
  my ($this, $account, $settings) = @_;
  croak 'usage CGP::CLI->SetWebUser($acount, \%settings)'
    unless defined $account && defined $settings;
  my $line = 'SetWebUser '.$account.' '.$this->printWords($settings);

  $this->send($line);
  $this->_parseResponse();
}

sub GetEffectiveWebUser {
  my ($this, $account) = @_;
  croak 'usage CGP::CLI->GetEffectiveWebUser($account)'
    unless defined $account;
  $this->send('GetEffectiveWebUser '.$account);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

#################################################################
#  Group managent commands

sub ListGroups {
  my ($this, $domainName) = @_;
  my $line = 'ListGroups';
  $line .= ' '.$domainName if $domainName;
  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub CreateGroup {
  my ($this, $groupName, $params) = @_;
  croak 'usage CGP::CLI->CreateGroup($groupName[, \%params])'
    unless defined $groupName;
  my $line = 'CreateGroup '.$groupName;
  $line .= ' '.$this->printWords($params) if $params;
  $this->send($line);
  $this->_parseResponse();
}

sub RenameGroup {
  my ($this, $oldGroupName, $newGroupName) = @_;
  croak 'usage CGP::CLI->RenameGroup($oldGroupName, $newGroupName)'
    unless defined $oldGroupName && defined $newGroupName;
  $this->send('RenameGroup '.$oldGroupName.' into '.$newGroupName);
  $this->_parseResponse();
}

sub DeleteGroup {
  my ($this, $groupName) = @_;
  croak 'usage CGP::CLI->DeleteGroup($groupName)'
    unless defined $groupName;
  $this->send('DeleteGroup '.$groupName);
  $this->_parseResponse();
}

sub GetGroup {
  my ($this, $groupName) = @_;
  croak 'usage CGP::CLI->GetGroup($groupName)'
    unless defined $groupName;
  $this->send('GetGroup '.$groupName);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetGroup {
  my ($this, $groupName, $params) = @_;
  croak 'usage CGP::CLI->SetGroup($groupName, \%settings)'
    unless defined $groupName && defined $params;
  $this->send('SetGroup '.$groupName.' '.$this->printWords($params));
  $this->_parseResponse();
}


#################################################################
#  Forwarder managent commands

sub ListForwarders {
  my ($this, $domainName) = @_;
  my $line = 'ListForwarders';
  $line .= ' ' .$domainName if $domainName;
  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub CreateForwarder {
  my ($this, $forwarderName, $address) = @_;
  croak 'usage CGP::CLI->CreateForwarder($forwarderName, $address)'
    unless defined $forwarderName && defined $address;
  $this->send('CreateForwarder '.$forwarderName.' TO '.$this->printWords($address));
  $this->_parseResponse();
}

sub DeleteForwarder {
  my ($this, $forwarderName) = @_;
  croak 'usage CGP::CLI->DeleteForwarder($forwarderName)'
    unless defined $forwarderName;
  $this->send('DeleteForwarder '.$forwarderName);
  $this->_parseResponse();
}

sub GetForwarder {
  my ($this, $forwarderName) = @_;
  croak 'usage CGP::CLI->GetForwarder($forwarderName)'
    unless defined $forwarderName;
  $this->send('GetForwarder '.$forwarderName);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

#################################################################
#  Domain managent commands

sub ListDomains {
  my $this = shift;
  $this->send('ListDomains');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub MainDomainName {
  my $this = shift;
  $this->send('MainDomainName');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetDomainSettings {
  my ($this, $domainName) = @_;

  my $line = 'GetDomainSettings';
  $line .= ' '.$domainName if($domainName);

  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetDomain {
  GetDomainSettings(@_);
}

sub GetDomainEffectiveSettings {
  my ($this, $domainName) = @_;
  my $line = 'GetDomainEffectiveSettings';
  $line .= ' '.$domainName if($domainName);
  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub UpdateDomainSettings {
  my ($this) = shift;
  my %params = (@_);

  croak 'usage CGP::CLI->UpdateDomainSettings(domain => \'domainName\',settings => {settings})'
    unless exists $params{settings};

  my $line = 'UpdateDomainSettings';
  $line .= ' '.$params{domain} if exists $params{domain};
  $this->send($line.' '.$this->printWords($params{settings}));
  $this->_parseResponse();
}

sub UpdateDomain {
  UpdateDomainSettings(@_);
}

sub SetDomainSettings {
  my ($this) = shift;
  my %params = (@_);

  croak 'usage CGP::CLI->SetDomainSettings(domain => \'domainName\',settings => {settings})'
    unless exists $params{settings};

  my $line = 'SetDomainSettings';
  $line .= ' '.$params{domain} if exists $params{domain};
  $this->send($line.' '.$this->printWords($params{settings}));
  $this->_parseResponse();
}

sub SetDomain {
  SetDomainSettings(@_);
}


sub CreateDomain {
  my ($this, $domainName, $params) = @_;
  croak 'usage CGP::CLI->CreateDomain($domainName[, \%params])'
    unless defined $domainName;
  my $line = 'CreateDomain '.$domainName;
  $line .= ' '.$this->printWords($params) if $params;
  $this->send($line);
  $this->_parseResponse();
}

sub RenameDomain {
  my ($this, $oldDomainName, $newDomainName) = @_;
  croak 'usage CGP::CLI->RenameDomain($oldDomainName, $newDomainName)'
    unless defined $oldDomainName && defined $newDomainName;
  $this->send('RenameDomain '.$oldDomainName.' into '.$newDomainName );
  $this->_parseResponse();
}

sub DeleteDomain {
  my ($this, $domainName, $force) = @_;
  croak 'usage CGP::CLI->DeleteDomain($domainName[, $force])'
    unless defined $domainName;
  my $line = 'DeleteDomain '.$domainName;
  $line .= ' force' if $force;
  $this->send($line);
  $this->_parseResponse();
}

sub CreateSharedDomain {
  my ($this, $domainName, $params) = @_;
  croak 'usage CGP::CLI->CreateSharedDomain($domainName[, \%params])'
    unless defined $domainName;
  my $line = 'CreateSharedDomain '.$domainName;
  $line .= ' '.$this->printWords($params) if $params;
  $this->send($line);
  $this->_parseResponse();
}

sub CreateDirectoryDomain {
  my ($this, $domainName, $params) = @_;
  croak 'usage CGP::CLI->CreateDirectoryDomain($domainName[, \%params])'
    unless defined $domainName;
  my $line = 'CreateDirectoryDomain '.$domainName;
  $line .= ' '.$this->printWords($params) if $params;
  $this->send($line);
  $this->_parseResponse();
}

sub ReloadDirectoryDomains {
  my ($this) = @_;
  $this->send('ReloadDirectoryDomains');
  $this->_parseResponse();
}

sub GetDomainRules {
  my ($this, $domainName) = @_;
  croak 'usage CGP::CLI->GetDomainRules($domainName)'
    unless defined $domainName;
  $this->send('GetDomainRules '.$domainName);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetDomainRules {
  my ($this, $domainName, $rules) = @_;
  croak 'usage CGP::CLI->SetDomainRules($domainName, \@rules)'
    unless defined $domainName && defined $rules;
  $this->send('SetDomainRules '.$domainName.' '.$this->printWords($rules));
  $this->_parseResponse();
}

sub GetDomainAliases {
  my ($this, $domain) = @_;

  my $line = 'GetDomainAliases';
  $line .= ' '.$domain if $domain;
  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}


sub SetDomainAliases {
  my ($this, $domain, $aliases) = @_;
  croak 'usage CGP::CLI->SetDomainAliases($domainName, \@aliases)'
    unless defined $domain && defined $aliases;
  $this->send('SetDomainAliases '.$domain.' '.$this->printWords($aliases));
  $this->_parseResponse();
}

sub ListAdminDomains {
  my ($this, $domain) = @_;
  my $line = 'ListAdminDomains';
  $line .= ' '.$domain if $domain;
  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub InsertDirectoryRecords {
  my ($this, $domain) = @_;
  my $line = 'InsertDirectoryRecords';
  $line .= ' '.$domain if $domain;
  $this->send($line);
  $this->_parseResponse();
}
sub DeleteDirectoryRecords {
  my ($this, $domain) = @_;
  my $line = 'DeleteDirectoryRecords';
  $line .= ' '.$domain if $domain;
  $this->send($line);
  $this->_parseResponse();
}


sub GetDirectoryIntegration {
  my $this = shift;
  $this->send('GetDirectoryIntegration');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetDirectoryIntegration {
  my ( $this, $dict ) = @_;
  croak 'usage CGP::CLI->SetDirectoryIntegration(\%settings)'
    unless defined $dict;

  $this->send('SetDirectoryIntegration '.$this->printWords($dict));
  $this->_parseResponse();
}

sub GetClusterDirectoryIntegration {
  my $this = shift;
  $this->send('GetClusterDirectoryIntegration');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetClusterDirectoryIntegration {
  my ( $this, $dict ) = @_;
  croak 'usage CGP::CLI->SetClusterDirectoryIntegration(\%settings)'
    unless defined $dict;

  $this->send('SetClusterDirectoryIntegration '.$this->printWords($dict));
  $this->_parseResponse();
}


sub GetDomainDefaults {
  my $this = shift;
  $this->send('GetDomainDefaults');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub UpdateDomainDefaults {
  my ( $this, $dict ) = @_;
  croak 'usage CGP::CLI->UpdateDomainDefaults(\%settings)'
    unless defined $dict;

  $this->send('UpdateDomainDefaults '.$this->printWords($dict));
  $this->_parseResponse();
}

sub SetDomainDefaults {
  my ( $this, $dict ) = @_;
  croak 'usage CGP::CLI->SetDomainDefaults(\%settings)'
    unless defined $dict;
  $this->send('SetDomainDefaults '.$this->printWords($dict));
  $this->_parseResponse();
}

sub GetClusterDomainDefaults {
  my $this = shift;
  $this->send('GetClusterDomainDefaults');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub UpdateClusterDomainDefaults {
  my ( $this, $dict ) = @_;
  $this->send('UpdateClusterDomainDefaults '.$this->printWords($dict));
  $this->_parseResponse();
}

sub SetClusterDomainDefaults {
  my ( $this, $dict ) = @_;
  $this->send('SetClusterDomainDefaults '.$this->printWords($dict));
  $this->_parseResponse();
}


sub GetAllAccountsDefaults {
  my $this = shift;
  $this->send('GetAllAccountsDefaults');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub UpdateAllAccountsDefaults {
  my ( $this, $dict ) = @_;
  $this->send('UpdateAllAccountsDefaults '.$this->printWords($dict));
  $this->_parseResponse();
}

sub SetAllAccountsDefaults {
  my ( $this, $dict ) = @_;
  $this->send('SetAllAccountsDefaults '.$this->printWords($dict));
  $this->_parseResponse();
}

sub GetClusterAccountDefaults {
  my $this = shift;
  $this->send('GetClusterAccountDefaults');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub UpdateClusterAccountDefaults {
  my ( $this, $dict ) = @_;
  $this->send('UpdateClusterAccountDefaults '.$this->printWords($dict));
  $this->_parseResponse();
}

sub SetClusterAccountDefaults {
  my ( $this, $dict ) = @_;
  $this->send('SetClusterAccountDefaults '.$this->printWords($dict));
  $this->_parseResponse();
}

sub GetServerWebUserDefaults {
  my $this = shift;
  $this->send('GetServerWebUserDefaults');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}
sub SetServerWebUserDefaults {
  my ( $this, $dict ) = @_;
  $this->send('SetServerWebUserDefaults '.$this->printWords($dict));
  $this->_parseResponse();
}

sub GetClusterWebUserDefaults {
  my $this = shift;
  $this->send('GetClusterWebUserDefaults');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}
sub SetClusterWebUserDefaults {
  my ( $this, $dict ) = @_;
  $this->send('SetClusterWebUserDefaults '.$this->printWords($dict));
  $this->_parseResponse();
}

sub GetDomainLocation {
  my ( $this, $account ) = @_;
  $this->send('GetDomainLocation '.$account);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetAccountLocation {
  my ( $this, $account ) = @_;
  $this->send('GetAccountLocation '.$account);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetAccountDefaults {
  my ($this, $domain) = @_;

  my $line = 'GetAccountDefaults';
  $line .= ' '.$domain if($domain);
  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub UpdateAccountDefaults {
  my ($this) = shift;
  my %params = (@_);

  croak 'usage CGP::CLI->UpdateAccountDefaults(domain => \'domainName\',settings => {settings})'
    unless exists $params{settings};

  my $line = 'UpdateAccountDefaults';
  $line .= ' '.$params{domain} if exists $params{domain};

  $this->send($line . ' ' . $this->printWords($params{settings}));
  $this->_parseResponse();
}

sub SetAccountDefaults {
  my ($this) = shift;
  my %params = (@_);

  croak 'usage CGP::CLI->SetAccountDefaults(domain => \'domainName\',settings => {settings})'
    unless exists $params{settings};

  my $line = 'SetAccountDefaults';
  $line .= ' '.$params{domain} if exists $params{domain};

  $this->send($line . ' ' . $this->printWords($params{settings}));
  $this->_parseResponse();
}

sub GetWebUserDefaults {
  my ($this, $domain) = @_;

  my $line = 'GetWebUserDefaults';
  $line .= ' '.$domain if($domain);

  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetWebUserDefaults {
  my ($this) = shift;;
  my %params = (@_);
  croak 'usage CGP::CLI->SetWebUserDefaults(domain => \'domainName\',settings => {settings})'
    unless exists $params{settings};

  my $line = 'SetWebUserDefaults';
  $line .= ' ' .$params{domain} if exists $params{domain};

  $this->send($line . ' ' . $this->printWords($params{settings}));
  $this->_parseResponse();
}


sub GetAccountTemplate {
  my ($this, $domain) = @_;

  my $line = 'GetAccountTemplate';
  $line .= ' '.$domain if($domain);

  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub UpdateAccountTemplate {
  my ($this) = shift;;
  my %params = (@_);

  croak 'usage CGP::CLI->UpdateAccountTemplate(domain => \'domainName\',settings => {settings})'
    unless exists $params{settings};

  my $line = 'UpdateAccountTemplate';
  $line .= ' '.$params{domain} if exists $params{domain};

  $this->send($line.' '.$this->printWords($params{settings}));
  $this->_parseResponse();
}

sub SetAccountTemplate {
  my ($this) = shift;;
  my %params = (@_);

  croak 'usage CGP::CLI->SetAccountTemplate(domain => \'domainName\',settings => {settings})'
    unless exists $params{settings};

  my $line = 'SetAccountTemplate';
  $line .= ' '.$params{domain} if exists $params{domain};

  $this->send($line.' '.$this->printWords($params{settings}));
  $this->_parseResponse();
}

#################################################################
#  Mailbox managent commands

sub ListMailboxes {
  my ($this) = shift;;
  my %params = (@_);

  croak 'usage CGP::CLI->ListMailboxes(accountName => \'account\',filter => \'*\',authAccountName => \'extAccount\')'
      unless exists $params{accountName};

  my $line = 'ListMailboxes '.$params{accountName};
  $line .= ' FILTER '.$this->printWords($params{filter}) if exists $params{filter};
  $line .= ' AUTH '.$params{authAccountName} if exists $params{authAccountName};

  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub CreateMailbox {
  my ($this,$accountName,$mailboxName,$authAccountName) = @_;


  croak 'usage CGP::CLI->CreateMailbox(accountName,mailboxName[,authAccountName])'
      unless (defined $accountName && defined $mailboxName);

  my $line = 'CREATEMAILBOX '.$accountName;
  $line .= ' MAILBOX '.$this->printWords($mailboxName);
  $line .= ' AUTH '.$authAccountName if defined $authAccountName;

  $this->send($line);
  $this->_parseResponse();
}

sub RenameMailbox {
  my ($this,$accountName,$oldMailboxName,$newMailboxName,$authAccountName) = @_;

  croak 'usage CGP::CLI->RenameMailbox(accountName,oldMailboxName,newMailboxName[,authAccountName])'
      unless (defined $accountName && defined $oldMailboxName && defined $newMailboxName);

  my $line = 'RENAMEMAILBOX '.$accountName;
  $line .= ' MAILBOX '.$this->printWords($oldMailboxName);
  $line .= ' INTO '.$this->printWords($newMailboxName);
  $line .= ' AUTH '.$authAccountName if defined $authAccountName;

  $this->send($line);
  $this->_parseResponse();
}

sub RenameMailboxes {
  my ($this,$accountName,$oldMailboxName,$newMailboxName,$authAccountName) = @_;

  croak 'usage CGP::CLI->RenameMailboxes(accountName,oldMailboxName,newMailboxName[,authAccountName])'
      unless (defined $accountName && defined $oldMailboxName && defined $newMailboxName);

  my $line = 'RENAMEMAILBOX '.$accountName;
  $line .= ' MAILBOXES '.$this->printWords($oldMailboxName);
  $line .= ' INTO '.$this->printWords($newMailboxName);
  $line .= ' AUTH '.$authAccountName if defined $authAccountName;

  $this->send($line);
  $this->_parseResponse();
}

sub DeleteMailbox {
  my ($this,$accountName,$mailboxName,$authAccountName) = @_;

  croak 'usage CGP::CLI->DeleteMailbox(accountName,mailboxName[,authAccountName])'
      unless (defined $accountName && defined $mailboxName);

  my $line = 'DELETEMAILBOX '.$accountName;
  $line .= ' MAILBOX '.$this->printWords($mailboxName);
  $line .= ' AUTH '.$authAccountName if defined $authAccountName;

  $this->send($line);
  $this->_parseResponse();
}

sub DeleteMailboxes {
  my ($this,$accountName,$mailboxName,$authAccountName) = @_;

  croak 'usage CGP::CLI->DeleteMailbox(accountName,mailboxName[,authAccountName])'
      unless (defined $accountName && defined $mailboxName);

  my $line = 'DELETEMAILBOX '.$accountName;
  $line .= ' MAILBOXES '.$this->printWords($mailboxName);
  $line .= ' AUTH '.$authAccountName if defined $authAccountName;

  $this->send($line);
  $this->_parseResponse();
}

sub GetMailboxInfo {
  my ($this,$accountName,$mailboxName,$authAccountName) = @_;

  croak 'usage CGP::CLI->GetMailboxInfo(accountName,mailboxName[,authAccountName])'
       unless (defined $accountName && defined $mailboxName);

  my $line = 'GETMAILBOXINFO '.$accountName;
  $line .= ' MAILBOX '.$this->printWords($mailboxName);
  $line .= ' AUTH '.$authAccountName if defined $authAccountName;

  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetMailboxACL {
  my ($this,$accountName,$mailboxName,$authAccountName) = @_;

  croak 'usage CGP::CLI->GetMailboxACL(accountName,mailboxName[,authAccountName])'
       unless (defined $accountName && defined $mailboxName);

  my $line = 'GETMAILBOXACL '.$accountName;
  $line .= ' MAILBOX '.$this->printWords($mailboxName);
  $line .= ' AUTH '.$authAccountName if defined $authAccountName;

  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetMailboxACL {
  my ($this,$accountName,$mailboxName,$newACL,$authAccountName) = @_;

  croak 'usage CGP::CLI->SetMailboxACL(accountName,mailboxName,\@newACL[,authAccountName])'
       unless (defined $accountName && defined $mailboxName && defined $newACL);

  my $line = 'SETMAILBOXACL '.$accountName;
  $line .= ' MAILBOX '.$this->printWords($mailboxName);
  $line .= ' AUTH '.$authAccountName if defined $authAccountName;

  $this->send($line.' '.$this->printWords($newACL));
  $this->_parseResponse();
}


sub GetMailboxRights {
  my ($this,$accountName,$mailboxName,$authAccountName) = @_;

  croak 'usage CGP::CLI->GetMailboxRights(accountName,mailboxName,authAccountName)'
       unless (defined $accountName && defined $mailboxName && defined $authAccountName);

  my $line = 'GETMAILBOXRIGHTS '.$accountName;
  $line .= ' MAILBOX '.$this->printWords($mailboxName);
  $line .= ' AUTH '.$authAccountName;

  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetMailboxClass {
  my ($this,$accountName,$mailboxName,$class,$authAccountName) = @_;

  croak 'usage CGP::CLI->SetMailboxClass(accountName,mailboxName,class [,authAccountName])'
       unless (defined $accountName && defined $mailboxName && defined $class);

  my $line = 'SetMailboxClass '.$accountName;
  $line .= ' MAILBOX '.$this->printWords($mailboxName);
  $line .= ' AUTH '.$authAccountName if(defined $authAccountName);
  $line .= ' CLASS '.$class;

  $this->send($line);
  return undef unless $this->_parseResponse();
}


sub GetAccountSubscription {
  my ($this,$accountName) = @_;
  croak 'usage CGP::CLI->GetAccountSubscription(accountName)'
       unless defined $accountName;

  $this->send('GetAccountSubscription '.$accountName);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetAccountSubscription {
  my ($this,$accountName,$newSubscription) = @_;
  croak 'usage CGP::CLI->SetAccountSubscription(accountName,\%newSubscription)'
       unless (defined $accountName && defined $newSubscription);

  $this->send('SetAccountSubscription '.$accountName.' '.$this->printWords($newSubscription));
  $this->_parseResponse();
}

sub GetMailboxAliases {
  my ($this,$accountName) = @_;
  croak 'usage CGP::CLI->GetMailboxAliases(accountName)'
       unless defined $accountName;

  $this->send('GetMailboxAliases '.$accountName);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetMailboxAliases {
  my ($this,$accountName,$newAliases) = @_;
  croak 'usage CGP::CLI->SetMailboxAliases(accountName,\%newAliases)'
       unless (defined $accountName && defined $newAliases);

  $this->send('SetMailboxAliases '.$accountName.' '.$this->printWords($newAliases));
  $this->_parseResponse();
}


#################################################################
#   Alerts Administration

sub GetDomainAlerts {
  my ($this, $domain) = @_;

  my $line = 'GetDomainAlerts';
  $line .= ' '.$domain if $domain;
  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetDomainAlerts {
  my ($this, $domain, $alerts) = @_;
  croak 'usage CGP::CLI->SetDomainAlerts($domainName, \@alerts)'
    unless defined $domain && defined $alerts;
  $this->send('SetDomainAlerts '.$domain.' '.$this->printWords($alerts));
  $this->_parseResponse();
}

sub PostDomainAlert {
  my ($this, $domain, $alert) = @_;
  croak 'usage CGP::CLI->PostDomainAlert($domainName, $alertString)'
    unless defined $domain && defined $alert;
  $this->send('PostDomainAlert '.$domain.' ALERT '.$this->printWords($alert));
  $this->_parseResponse();
}

sub RemoveDomainAlert {
  my ($this, $domain, $timeStamp) = @_;
  croak 'usage CGP::CLI->RemoveDomainAlert($domainName, $timeStamp)'
    unless defined $domain && defined $timeStamp;
  $this->send('RemoveDomainAlert '.$domain.' ALERT '.$timeStamp);
  $this->_parseResponse();
}

sub GetAccountAlerts {
  my ($this, $account) = @_;
  $this->send('GetAccountAlerts '.$account);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetAccountAlerts {
  my ($this, $account, $alerts) = @_;
  croak 'usage CGP::CLI->SetAccountAlerts($account, \@alerts)'
    unless defined $account && defined $alerts;
  $this->send('SetAccountAlerts '.$account.' '.$this->printWords($alerts));
  $this->_parseResponse();
}

sub PostAccountAlert {
  my ($this, $account, $alert) = @_;
  croak 'usage CGP::CLI->PostAccountAlert($account, $alertString)'
    unless defined $account && defined $alert;
  $this->send('PostAccountAlert '.$account.' ALERT '.$this->printWords($alert));
  $this->_parseResponse();
}

sub RemoveAccountAlert {
  my ($this, $account, $timeStamp) = @_;
  croak 'usage CGP::CLI->RemoveAccountAlert($account, $timeStamp)'
    unless defined $account && defined $timeStamp;
  $this->send('RemoveAccountAlert '.$account.' ALERT '.$timeStamp);
  $this->_parseResponse();
}


sub GetServerAlerts {
  my ($this) = @_;
  $this->send('GetServerAlerts');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetServerAlerts {
  my ($this, $alerts) = @_;
  croak 'usage CGP::CLI->SetServerAlerts(\@alerts)'
    unless defined $alerts;
  $this->send('SetServerAlerts '.$this->printWords($alerts));
  $this->_parseResponse();
}

sub PostServerAlert {
  my ($this,  $alert) = @_;
  croak 'usage CGP::CLI->PostServerAlert($alertString)'
    unless defined $alert;
  $this->send('PostServerAlert '.$this->printWords($alert));
  $this->_parseResponse();
}

sub RemoveServerAlert {
  my ($this, $timeStamp) = @_;
  croak 'usage CGP::CLI->RemoveServerAlert($timeStamp)'
    unless defined $timeStamp;
  $this->send('RemoveServerAlert '.$timeStamp);
  $this->_parseResponse();
}

sub GetClusterAlerts {
  my ($this) = @_;
  $this->send('GetClusterAlerts');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetClusterAlerts {
  my ($this, $alerts) = @_;
  croak 'usage CGP::CLI->SetClusterAlerts(\@alerts)'
    unless defined $alerts;
  $this->send('SetClusterAlerts '.$this->printWords($alerts));
  $this->_parseResponse();
}

sub PostClusterAlert {
  my ($this,  $alert) = @_;
  croak 'usage CGP::CLI->PostClusterAlert($alertString)'
    unless defined $alert;
  $this->send('PostClusterAlert '.$this->printWords($alert));
  $this->_parseResponse();
}

sub RemoveClusterAlert {
  my ($this, $timeStamp) = @_;
  croak 'usage CGP::CLI->RemoveClusterAlert($timeStamp)'
    unless defined $timeStamp;
  $this->send('RemoveClusterAlert '.$timeStamp);
  $this->_parseResponse();
}

#################################################################
#   Personal Web Site Administration

sub GetWebFile {
  my ($this,$accountName,$fileName,$position,$sliceSize) = @_;

  croak 'usage CGP::CLI->GetWebFile(accountName,fileName[,position,sliceSize])'
       unless (defined $accountName && defined $fileName);
  my $line='GetWebFile '.$accountName.' FILE '.$this->printWords($fileName);
  $line .= ' OFFSET '. $position if(defined $position && $position!=0);
  $line .= ' SIZE '. $sliceSize if(defined $sliceSize);

  $this->send($line);
  return undef unless $this->_parseResponse;
  $this->parseWords($this->getWords);
}

sub PutWebFile {
  my ($this,$accountName,$fileName,$data,$position) = @_;

  croak 'usage CGP::CLI->PutWebFile(accountName,fileName,data [,position])'
       unless (defined $accountName && defined $fileName && defined $data);
  my $line='PutWebFile '.$accountName.' FILE '.$this->printWords($fileName);
  $line .= ' OFFSET '. $position if(defined $position && $position!=0);

  $line .= ' DATA "'.$data.'"';

  $this->send($line);
  $this->_parseResponse;
}

sub RenameWebFile {
  my ($this,$accountName,$oldFileName,$newFileName) = @_;

  croak 'usage CGP::CLI->RenameWebFile(accountName,oldFileName,newFileName)'
       unless (defined $accountName && defined $oldFileName && defined $newFileName);
  $this->send('RenameWebFile '.$accountName.' FILE '.$this->printWords($oldFileName).' INTO '.$this->printWords($newFileName));
  $this->_parseResponse;
}

sub DeleteWebFile {
  my ($this,$accountName,$fileName) = @_;

  croak 'usage CGP::CLI->DeleteWebFile(accountName,fileName)'
       unless (defined $accountName && defined $fileName);
  $this->send('DeleteWebFile '.$accountName.' FILE '.$this->printWords($fileName));
  $this->_parseResponse;
}

sub ListWebFiles {
  my ($this,$accountName,$filePath) = @_;

  croak 'usage CGP::CLI->ListWebFiles(accountName [,filePath])'
       unless defined $accountName;
  my $line = 'ListWebFiles '.$accountName;
  $line .= ' PATH '.$this->printWords($filePath) if $filePath;
  $this->send($line);
  return undef unless $this->_parseResponse;
  $this->parseWords($this->getWords);
}

sub GetWebFilesInfo {
  my ($this,$accountName) = @_;

  croak 'usage CGP::CLI->InfoWebFiles(accountName)'
       unless defined $accountName;
  $this->send('GetWebFilesInfo '.$accountName);
  return undef unless $this->_parseResponse;
  $this->parseWords($this->getWords);
}

#################################################################
#   List management commands

sub ListLists {
  my ( $this, $domain ) = @_;
  my $line = 'LISTLISTS';
  $line .= ' '.$domain if $domain;
  $this->send($line);
  return undef unless $this->_parseResponse;
  $this->parseWords($this->getWords);
}

sub GetDomainLists {
  my ( $this, $domain ) = @_;
  my $line = 'GetDomainLists';
  $line .= ' '.$domain if $domain;
  $this->send($line);
  return undef unless $this->_parseResponse;
  $this->parseWords($this->getWords);
}

sub GetAccountLists {
  my ( $this, $accountName ) = @_;
  croak 'usage CGP::CLI->GetAccoountLists($accountName)'
      unless defined $accountName;

  $this->send('GetAccountLists '.$accountName);
  return undef unless $this->_parseResponse;
  $this->parseWords($this->getWords);
}

sub CreateList {
  my ($this, $listName, $accountName) = @_;
  croak 'usage CGP::CLI->CreateList($listName, $accountName)'
      unless defined $listName && defined $accountName;
  $this->send('CREATELIST '.$listName.' for '.$accountName);
  $this->_parseResponse();
}

sub RenameList {
  my ($this, $oldListName, $newListName) = @_;
  croak 'usage CGP::CLI->RenameList($oldListName, $newListName)'
      unless defined $oldListName && defined $newListName;
  $this->send('RENAMELIST '.$oldListName.' into '.$newListName);
  $this->_parseResponse();
}

sub DeleteList {
  my ( $this, $listName ) = @_;
  croak 'usage CGP::CLI->DeleteList($listName)'
      unless defined $listName;

  $this->send('DELETELIST '.$listName);
  $this->_parseResponse();
}

sub GetList {
  my ( $this, $listName ) = @_;
  croak 'usage CGP::CLI->GetList($listName)'
      unless defined $listName;

  $this->send('GETLIST '.$listName);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub UpdateList {
  my ( $this, $listName, $dict ) = @_;
  croak 'usage CGP::CLI->UpdateList($listName,\%params)'
      unless defined $listName && defined $dict;

  $this->send('UPDATELIST '.$listName.' '.$this->printWords($dict) );
  $this->_parseResponse();
}

sub List {
  my ( $this, $listName, $command, $subscriber, @options ) = @_;
  croak 'usage CGP::CLI->List($listName, $command, $subscriber, @options)'
      unless defined $listName && defined $command && defined $subscriber;

  $this->send('LIST '.$listName.' '.$command.' '.join(' ',@options).' '.$subscriber);
  $this->_parseResponse();
}

sub ListSubscribers {
  my ( $this, $listName,$filter,$limit ) = @_;
  croak 'usage CGP::CLI->ListSubscribers($listName [,$filter[,$limit]])'
      unless defined $listName;

  my $line = 'ListSubscribers '.$listName;
  $line .= ' FILTER '. ($filter eq '' ? '""' : $this->printWords($filter)) if($filter || $limit);
  $line .= " $limit" if($limit);

  $this->send($line );
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetSubscriberInfo {
  my ( $this, $listName,$address ) = @_;
  croak 'usage CGP::CLI->GetSubscriberInfo($listName ,$subscriberAddress)'
      unless defined $listName && defined $address;
  $this->send('GetSubscriberInfo '.$listName.' NAME '.$address);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetPostingMode {
  my ($this, $listName, $address, $mode) = @_;
  croak 'usage CGP::CLI->SetPostingMode($listName,$subscriberAddress,$mode)'
      unless defined $listName && defined $address && defined $mode;
  $this->send('SetPostingMode '.$listName.' FOR '.$address.' '.$mode);
  $this->_parseResponse();
}

sub ProcessBounce {
  my ($this, $listName, $address, $fatal) = @_;
  croak 'usage CGP::CLI->ProcessBounce($listName,$subscriberAddress,"fatal")'
      unless defined $listName && defined $address;
  my $line = 'ProcessBounce '.$listName;
  $line .= ' FATAL' if($fatal);
  $this->send($line. ' FOR '.$address);
  $this->_parseResponse();
}

#############################################
#   Web Skins Administration

sub ListDomainSkins {
  my ($this, $domainName) = @_;
  my $line = 'ListDomainSkins';
  $line .= ' '.$domainName if($domainName);
  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub CreateDomainSkin {
  my ( $this, $domain, $skin ) = @_;
  croak 'usage CGP::CLI->CreateDomainSkin($domainName,$skinName)'
      unless defined $domain && defined $skin;

  $this->send('CreateDomainSkin '.$domain.' SKIN '.$this->printWords($skin));
  $this->_parseResponse();
}

sub RenameDomainSkin {
  my ( $this, $domain, $skin, $newName ) = @_;
  croak 'usage CGP::CLI->RenameDomainSkin($domainName,$oldSkinName,$newSkinName)'
      unless defined $domain && defined $skin && defined $newName;

  $this->send('RenameDomainSkin '.$domain.' SKIN '.$this->printWords($skin).' INTO '.$this->printWords($newName));
  $this->_parseResponse();
}

sub DeleteDomainSkin {
  my ( $this, $domain, $skin ) = @_;
  croak 'usage CGP::CLI->DeleteDomainSkin($domainName,$skinName)'
      unless defined $domain && defined $skin;

  $this->send('DeleteDomainSkin '.$domain.' SKIN '.$this->printWords($skin));
  $this->_parseResponse();
}

sub ListDomainSkinFiles {
  my ( $this, $domain, $skin ) = @_;
  croak 'usage CGP::CLI->ListDomainSkinFiles($domainName,$skinName)'
      unless defined $domain && defined $skin;

  $this->send('ListDomainSkinFiles '.$domain.' SKIN '.$this->printWords($skin));
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub ReadDomainSkinFile {
  my ( $this, $domain, $skin, $fileName ) = @_;
  croak 'usage CGP::CLI->ReadDomainSkinFile($domainName,$skinName,$fileName)'
      unless defined $domain && defined $skin && defined $fileName;

  $this->send('ReadDomainSkinFile '.$domain.' SKIN '.$this->printWords($skin).' FILE '.$this->printWords($fileName));
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub StoreDomainSkinFile {
  my ( $this, $domain, $skin, $fileName,$base64data ) = @_;
  croak 'usage CGP::CLI->StoreDomainSkinFile($domainName,$skinName,$fileName,$base64data)'
      unless defined $domain && defined $skin && defined $fileName && defined $base64data;
  $this->send('StoreDomainSkinFile '.$domain.' SKIN '.$this->printWords($skin).' FILE '.$this->printWords($fileName).' DATA "'.$base64data.'"');
  $this->_parseResponse();
}

sub DeleteDomainSkinFile {
  my ( $this, $domain, $skin, $fileName ) = @_;
  croak 'usage CGP::CLI->DeleteDomainSkinFile($domainName,$skinName,$fileName)'
      unless defined $domain && defined $skin && defined $fileName;

  $this->send('StoreDomainSkinFile '.$domain.' SKIN '.$this->printWords($skin).' FILE '.$this->printWords($fileName).' DELETE');
  $this->_parseResponse();
}

#--
sub ListServerSkins {
  my ($this) = @_;
  $this->send('ListServerSkins');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub CreateServerSkin {
  my ( $this, $skin ) = @_;
  croak 'usage CGP::CLI->CreateServerSkin($skinName)'
      unless defined $skin;

  $this->send('CreateServerSkin '.$this->printWords($skin));
  $this->_parseResponse();
}

sub RenameServerSkin {
  my ( $this, $skin, $newName ) = @_;
  croak 'usage CGP::CLI->RenameServerSkin($oldSkinName,$newSkinName)'
      unless defined $skin && defined $newName;

  $this->send('RenameServerSkin '.$this->printWords($skin).' INTO '.$this->printWords($newName));
  $this->_parseResponse();
}

sub DeleteServerSkin {
  my ( $this, $skin ) = @_;
  croak 'usage CGP::CLI->DeleteServerSkin($skinName)'
      unless defined $skin;

  $this->send('DeleteServerSkin '.$this->printWords($skin));
  $this->_parseResponse();
}

sub ListServerSkinFiles {
  my ( $this, $skin ) = @_;
  croak 'usage CGP::CLI->ListServerSkinFiles($skinName)'
      unless defined $skin;

  $this->send('ListServerSkinFiles '.$this->printWords($skin));
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub ReadServerSkinFile {
  my ( $this, $skin, $fileName ) = @_;
  croak 'usage CGP::CLI->ReadServerSkinFile($skinName,$fileName)'
      unless defined $skin && defined $fileName;

  $this->send('ReadServerSkinFile '.$this->printWords($skin).' FILE '.$this->printWords($fileName));
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub StoreServerSkinFile {
  my ( $this, $skin, $fileName,$data ) = @_;
  croak 'usage CGP::CLI->StoreServerSkinFile($skinName,$fileName,$base64data)'
      unless defined $skin && defined $fileName && defined $data;

  $this->send('StoreServerSkinFile '.$this->printWords($skin).' FILE '.$this->printWords($fileName).' DATA "'.$data.'"');
  $this->_parseResponse();
}

sub DeleteServerSkinFile {
  my ( $this, $skin, $fileName ) = @_;
  croak 'usage CGP::CLI->DeleteServerSkinFile($skinName,$fileName)'
      unless defined $skin && defined $fileName;

  $this->send('StoreServerSkinFile '.$this->printWords($skin).' FILE '.$this->printWords($fileName).' DELETE');
  $this->_parseResponse();
}

#--
sub ListClusterSkins {
  my ($this) = @_;
  $this->send('ListClusterSkins');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub CreateClusterSkin {
  my ( $this, $skin ) = @_;
  croak 'usage CGP::CLI->CreateClusterSkin($skinName)'
      unless defined $skin;

  $this->send('CreateClusterSkin '.$this->printWords($skin));
  $this->_parseResponse();
}

sub RenameClusterSkin {
  my ( $this, $skin, $newName ) = @_;
  croak 'usage CGP::CLI->RenameClusterSkin($oldSkinName,$newSkinName)'
      unless defined $skin && defined $newName;

  $this->send('RenameClusterSkin '.$this->printWords($skin).' INTO '.$this->printWords($newName));
  $this->_parseResponse();
}

sub DeleteClusterSkin {
  my ( $this, $skin ) = @_;
  croak 'usage CGP::CLI->DeleteClusterSkin($skinName)'
      unless defined $skin;

  $this->send('DeleteClusterSkin '.$this->printWords($skin));
  $this->_parseResponse();
}

sub ListClusterSkinFiles {
  my ( $this, $skin ) = @_;
  croak 'usage CGP::CLI->ListClusterSkinFiles($skinName)'
      unless defined $skin;

  $this->send('ListClusterSkinFiles '.$this->printWords($skin));
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub ReadClusterSkinFile {
  my ( $this, $skin, $fileName ) = @_;
  croak 'usage CGP::CLI->ReadClusterSkinFile($skinName,$fileName)'
      unless defined $skin && defined $fileName;

  $this->send('ReadClusterSkinFile '.$this->printWords($skin).' FILE '.$this->printWords($fileName));
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub StoreClusterSkinFile {
  my ( $this, $skin, $fileName,$data ) = @_;
  croak 'usage CGP::CLI->StoreClusterSkinFile($skinName,$fileName,$base64data)'
      unless defined $skin && defined $fileName && defined $data;

  $this->send('StoreClusterSkinFile '.$this->printWords($skin).' FILE '.$this->printWords($fileName).' DATA "'.$data.'"');
  $this->_parseResponse();
}

sub DeleteClusterSkinFile {
  my ( $this, $skin, $fileName ) = @_;
  croak 'usage CGP::CLI->DeleteClusterSkinFile($skinName,$fileName)'
      unless defined $skin && defined $fileName;

  $this->send('StoreClusterSkinFile '.$this->printWords($skin).' FILE '.$this->printWords($fileName).' DELETE');
  $this->_parseResponse();
}

#############################################
#   Web Interface Tuning

sub ListWebUserInterface {
  my ($this, $domainName, $path) = @_;

  my $line = 'ListWebUserInterface';
  $line .= ' '.$domainName if($domainName);
  $line .= ' PATH '.$this->printWords($path) if($path);

  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetWebUserInterface {
  my ($this, $domainName, $path) = @_;
  croak 'usage CGP::CLI->GetWebUserInterface($domainName, $fileName)'
    unless defined $domainName && defined $path;

  my $line = 'GetWebUserInterface '.$domainName.' FILE '.$this->printWords($path);
  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub PutWebUserInterface {
  my ($this, $domainName, $path, $data) = @_;
  croak 'usage CGP::CLI->PutWebUserInterface($domainName, $fileName, $base64data)'
    unless defined $domainName && defined $path && defined $data;

  my $line = 'PutWebUserInterface '.$domainName;
  $line .= ' FILE '.$this->printWords($path).' DATA "'.$data.'"';
  $this->send($line);
  $this->_parseResponse();
}

sub DeleteWebUserInterface {
  my ($this, $domainName, $path) = @_;
  croak 'usage CGP::CLI->DeleteWebUserInterface($domainName, $fileName)'
    unless defined $domainName && defined $path;

  my $line = 'DeleteWebUserInterface ';
  $line .= ' '.$domainName if($domainName);
  $line .= ' FILE '.$this->printWords($path) if($path);

  $this->send($line);
  $this->_parseResponse();
}

sub ClearWebUserCache {
  my ($this, $domainName) = @_;

  my $line = 'ClearWebUserCache';
  $line .= ' '.$domainName if($domainName);

  $this->send($line);
  $this->_parseResponse();
}

#############################################
#   Web Interface Integration

sub CreateWebUserSession {
  my ($this, $accountName, $ipAddress, $wml) = @_;
  croak 'usage CGP::CLI->CreateWebUserSession($accountName, $IP_Address,"WML")'
    unless defined $accountName && defined $ipAddress;
  my $line='CreateWebUserSession '.$accountName.' ADDRESS '.$ipAddress;
  $line .= ' WML' if($wml);
  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetWebUserSession {
  my ($this, $sessionID,$domain) = @_;
  croak 'usage CGP::CLI->GetWebUserSession($sessionID [,$domain])' unless defined $sessionID;

  my $line='GetWebUserSession '.$sessionID;
  $line .= ' DOMAIN '.$domain if($domain);

  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub KillWebUserSession {
  my ($this, $sessionID,$domain) = @_;
  croak 'usage CGP::CLI->KillWebUserSession($sessionID [,$domain])' unless defined $sessionID;
  my $line='KillWebUserSession '.$sessionID;
  $line .= ' DOMAIN '.$domain if($domain);

  $this->send($line);
  $this->_parseResponse();
}


#############################################
#  Server Settings

sub GetModule {
  my ( $this, $moduleName ) = @_;
  $this->send('GetModule '.$moduleName);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub UpdateModule {
  my ( $this, $moduleName, $newSettings ) = @_;
  croak 'usage CGP::CLI->UpdateModule($moduleName, \%newSettings)'
    unless defined $moduleName && defined $newSettings;
  $this->send ('UpdateModule '.$moduleName.' '.$this->printWords($newSettings) );
  $this->_parseResponse();
}

sub SetModule {
  my ( $this, $moduleName, $newSettings ) = @_;
  croak 'usage CGP::CLI->SetModule($moduleName, \%newSettings)'
    unless defined $moduleName && defined $newSettings;
  $this->send ('SetModule '.$moduleName.' '.$this->printWords($newSettings) );
  $this->_parseResponse();
}

sub GetLANIPs {
  my ( $this ) = @_;
  $this->send('GetLANIPs');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetBlacklistedIPs {
  my ( $this ) = @_;
  $this->send('GetBlacklistedIPs');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetClientIPs {
  my ( $this ) = @_;
  $this->send('GetClientIPs');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetWhiteHoleIPs {
  my ( $this ) = @_;
  $this->send('GetWhiteHoleIPs');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetProtection {
  my ( $this ) = @_;
  $this->send('GetProtection');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetBanned {
  my ( $this ) = @_;
  $this->send('GetBanned');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetLANIPs {
  my ( $this, $addresses ) = @_;
  croak 'usage CGP::CLI->SetLANIPs("10.0.0.1\e192.168.0.1")'
    unless defined $addresses;
  $this->send ('SetLANIPs '.$this->printWords($addresses));
  $this->_parseResponse();
}


sub SetBlacklistedIPs {
  my ( $this, $addresses ) = @_;
  croak 'usage CGP::CLI->SetBlacklistedIPs("11.22.33.44\e55.66.77.88")'
    unless defined $addresses;
  $this->send ('SetBlacklistedIPs '.$this->printWords($addresses));
  $this->_parseResponse();
}

sub SetClientIPs {
  my ( $this, $addresses ) = @_;
  croak 'usage CGP::CLI->SetClientIPs("11.22.33.44\e55.66.77.88")'
    unless defined $addresses;
  $this->send ('SetClientIPs '.$this->printWords($addresses));
  $this->_parseResponse();
}

sub SetWhiteHoleIPs {
  my ( $this, $addresses ) = @_;
  croak 'usage CGP::CLI->SetWhiteHoleIPs("11.22.33.44\e55.66.77.88")'
    unless defined $addresses;
  $this->send ('SetWhiteHoleIPs '.$this->printWords($addresses));
  $this->_parseResponse();
}

sub SetProtection {
  my ( $this, $settings ) = @_;
  croak 'usage CGP::CLI->SetProtection(\%settings)'
    unless defined $settings;
  $this->send ('SetProtection '.$this->printWords($settings));
  $this->_parseResponse();
}

sub SetBanned {
  my ( $this, $settings ) = @_;
  croak 'usage CGP::CLI->SetBanned(\%settings)'
    unless defined $settings;
  $this->send ('SetBanned '.$this->printWords($settings));
  $this->_parseResponse();
}

sub GetClusterLANIPs {
  my ( $this ) = @_;
  $this->send('GetClusterLANIPs');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetClusterBlacklistedIPs {
  my ( $this ) = @_;
  $this->send('GetClusterBlacklistedIPs');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetClusterClientIPs {
  my ( $this ) = @_;
  $this->send('GetClusterClientIPs');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetClusterWhiteHoleIPs {
  my ( $this ) = @_;
  $this->send('GetClusterWhiteHoleIPs');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub GetClusterProtection {
  my ( $this ) = @_;
  $this->send('GetClusterProtection');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}
sub GetClusterBanned {
  my ( $this ) = @_;
  $this->send('GetClusterBanned');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetClusterLANIPs {
  my ( $this, $addresses ) = @_;
  croak 'usage CGP::CLI->SetClusterLANIPs("10.0.0.1\e192.168.0.1")'
    unless defined $addresses;
  $this->send ('SetClusterLANIPs '.$this->printWords($addresses));
  $this->_parseResponse();
}

sub SetClusterBlacklistedIPs {
  my ( $this, $addresses ) = @_;
  croak 'usage CGP::CLI->SetClusterBlacklistedIPs("11.22.33.44\e55.66.77.88")'
    unless defined $addresses;
  $this->send ('SetClusterBlacklistedIPs '.$this->printWords($addresses));
  $this->_parseResponse();
}

sub SetClusterClientIPs {
  my ( $this, $addresses ) = @_;
  croak 'usage CGP::CLI->SetClusterClientIPs("11.22.33.44\e55.66.77.88")'
    unless defined $addresses;
  $this->send ('SetClusterClientIPs '.$this->printWords($addresses));
  $this->_parseResponse();
}

sub SetClusterWhiteHoleIPs {
  my ( $this, $addresses ) = @_;
  croak 'usage CGP::CLI->SetClusterWhiteHoleIPs("11.22.33.44\e55.66.77.88")'
    unless defined $addresses;
  $this->send ('SetClusterWhiteHoleIPs '.$this->printWords($addresses));
  $this->_parseResponse();
}

sub SetClusterProtection {
  my ( $this, $settings ) = @_;
  croak 'usage CGP::CLI->SetClusterProtection(\%settings)'
    unless defined $settings;
  $this->send ('SetClusterProtection '.$this->printWords($settings));
  $this->_parseResponse();
}

sub GetServerRules {
  my $this = shift;
  $this->send('GetServerRules');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetServerRules {
  my ($this, $rules) = @_;
  croak 'usage CGP::CLI->SetServerRules(\@rules)'
    unless defined $rules;
  $this->send('SetServerRules '.$this->printWords($rules));
  $this->_parseResponse();
}

sub GetClusterRules {
  my $this = shift;
  $this->send('GetClusterRules');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetClusterRules {
  my ($this, $rules) = @_;
  croak 'usage CGP::CLI->SetClusterRules(\@rules)'
    unless defined $rules;
  $this->send('SetClusterRules '.$this->printWords($rules));
  $this->_parseResponse();
}

sub RefreshOSData {
  my ($this) = @_;
  $this->send('RefreshOSData');
  $this->_parseResponse();
}

sub GetRouterTable {
  my $this = shift;
  $this->send('GetRouterTable');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetRouterTable {
  my ($this, $table) = @_;
  croak 'usage CGP::CLI->GetRouterTable("<addr1>=addr1\@domain.com\\e<addr2>=addr2\@domain.com\\e")'
    unless defined $table;
  $this->send('SetRouterTable '.$this->printWords($table));
  $this->_parseResponse();
}

sub GetRouterSettings {
  my $this = shift;
  $this->send('GetRouterSettings');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetRouterSettings {
  my ($this, $settings) = @_;
  croak 'usage CGP::CLI->SetRouterSettings(\@settings)'
    unless defined $settings;
  $this->send('SetRouterSettings '.$this->printWords($settings));
  $this->_parseResponse();
}


sub GetClusterRouterTable {
  my $this = shift;
  $this->send('GetClusterRouterTable');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub SetClusterRouterTable {
  my ($this, $table) = @_;
  croak 'usage CGP::CLI->SetClusterRouterTable("<addr1>=addr1\@domain.com\\e<addr2>=addr2\@domain.com\\e")'
    unless defined $table;
  $this->send('SetClusterRouterTable '.$this->printWords($table));
  $this->_parseResponse();
}


sub GetServerIntercept {
  my ( $this ) = @_;
  $this->send('GetServerIntercept');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}
sub SetServerIntercept {
  my ( $this, $settings ) = @_;
  croak 'usage CGP::CLI->SetServerIntercept(\%settings)'
    unless defined $settings;
  $this->send ('SetServerIntercept '.$this->printWords($settings));
  $this->_parseResponse();
}

sub GetClusterIntercept {
  my ( $this ) = @_;
  $this->send('GetClusterIntercept');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}
sub SetClusterIntercept {
  my ( $this, $settings ) = @_;
  croak 'usage CGP::CLI->SetClusterIntercept(\%settings)'
    unless defined $settings;
  $this->send ('SetClusterIntercept '.$this->printWords($settings));
  $this->_parseResponse();
}


sub Route {
  my ($this, $address, $mail) = @_;
  croak 'usage CGP::CLI->Route(address[,"mail"])'
    unless defined $address;
  my $line='Route '.$address;
  $line .= ' mail' if($mail);
  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

#############################################
#  Monitoring commands

sub GetSNMPElement {
  my ($this, $element) = @_;
  $this->send('GetSNMPElement '.$element);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub Shutdown {
  my $this = shift;
  $this->send('SHUTDOWN');
  $this->_parseResponse();
}

#############################################
#  Statistics commands

sub GetAccountStat {
  my ($this, $account, $key) = @_;
  croak 'usage CGP::CLI->GetAccountStat($account[,$key])'
    unless defined $account;
  my $line = 'GetAccountStat '.$account;
  $line .= ' Key '.$key if $key;
  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub ResetAccountStat {
  my ($this, $account, $key) = @_;
  croak 'usage CGP::CLI->ResetAccountStat($account[,$key])'
    unless defined $account;
  my $line = 'ResetAccountStat '.$account;
  $line .= ' Key '.$key if $key;
  $this->send($line);
  $this->_parseResponse();
}

sub GetDomainStat {
  my ($this, $domain, $key) = @_;
  croak 'usage CGP::CLI->GetDomainStat($domain[,$key])'
    unless defined $domain;
  my $line = 'GetDomainStat '.$domain;
  $line .= ' Key '.$key if $key;
  $this->send($line);
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub ResetDomainStat {
  my ($this, $domain, $key) = @_;
  croak 'usage CGP::CLI->ResetDomainStat($domain[,$key])'
    unless defined $domain;
  my $line = 'ResetDomainStat '.$domain;
  $line .= ' Key '.$key if $key;
  $this->send($line);
  $this->_parseResponse();
}

#############################################
#  Miscellaneous commands

sub WriteLog {
  my ($this, $level, $msg) = @_;
  croak 'usage CGP::CLI->WriteLog($level,$message)'
    unless defined $level && defined $msg;
  $this->send('WriteLog '.$level.' '.$this->printWords($msg));
  $this->_parseResponse();
}
sub ReleaseSMTPQueue {
  my ($this, $queue) = @_;
  croak 'usage CGP::CLI->ReleaseSMTPQueue($queueName)'
    unless defined $queue;
  $this->send('ReleaseSMTPQueue '.$this->printWords($queue));
  $this->_parseResponse();
}
sub RejectQueueMessage {
  my ($this, $msg, $text) = @_;
  croak 'usage CGP::CLI->RejectQueueMessage($msgID [,$errorText])'
    unless defined $msg;
  my $line = 'RejectQueueMessage '.$msg;
  $line .= $this->printWords($text) if $text;
  $this->send($line);
  $this->_parseResponse();
}

sub GetCurrentController {
  my ($this) = @_;
  $this->send('GetCurrentController');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}
sub GetTempClientIPs  {
  my ($this) = @_;
  $this->send('GetTempClientIPs');
  return undef unless $this->_parseResponse();
  $this->parseWords($this->getWords);
}

sub RemoveAccountSubset  {
  my ($this, $account, $subset) = @_;
  croak 'usage CGP::CLI->RemoveAccountSubset($account,$subset)'
    unless defined $account && defined $subset;
  $this->send('RemoveAccountSubset '.$account.' SUBSET '.$this->printWords($subset));
  return undef unless $this->_parseResponse();
}


#########################################################################
#########################################################################
##  Internal routines

sub _setStrangeError
{
    my ($this, $line, $code) = @_;
    if ($code)
    {
        ${*$this}{'errCode'} = $code;
    }
    else
    {
        ${*$this}{'errCode'} = $CGP::CLI_CODE_STRANGE;
    }
    ${*$this}{'errMsg'} = $line;
    return $CGP::FALSE;
}

sub _parseResponse
{
  my $this = shift;
  my $responseLine = <$this>;

  print STDERR "CGP::CLI->_parseResponse::responseLine = $responseLine\n\n"
      if ${*$this}{'debug'};

  $responseLine =~ /^(\d+)\s(.*)$/;
  return $this->_setStrangeError($responseLine) unless ($1);
  ${*$this}{'errCode'} = $1;
  if($1 == $CGP::CLI_CODE_OK_INLINE) {
    ${*$this}{'inlineResponse'} = $2;
    ${*$this}{'errMsg'} = 'OK';
  } else {
    ${*$this}{'errMsg'} = $2;
    chomp(${*$this}{'errMsg'});
    ${*$this}{'errMsg'} =~ s/\r$//;
  }
  $this->isSuccess;
}

sub convertOutput {
  my $data = $_[0];
  my $translate = $_[1];
  if(!defined($data)) {
    return '""';
  } elsif(ref($data) eq 'HASH') {
    my $outp='{';
    #$outp.="\n";
    foreach (sort keys %$data) {
      my $value=@$data{$_};
      $outp .= convertOutput($_,$translate).'='.convertOutput($value,$translate).';';
      #$outp.="\n";
    }
    $outp.= '}';
    #$outp.="\n";
    return $outp;
  } elsif(ref($data) eq 'ARRAY') {
    my $outp='(';
    my $first=1;
    foreach (@$data) {
      if(!$first) { $outp.=','; } else { $first=0; }
      $outp.=convertOutput($_,$translate);
    }
    $outp.= ')';
    return $outp;
  } else {
    if($data =~ /[\W_]/ || $data eq '') {
      if($translate) {
        $data =~ s/\\((?![enr\d]))/\\\\$1/g;
        $data =~ s/\"/\\\"/g;
      }
      $data =~ s/([\x00-\x1F\x7F])/'\\'.('0'x(3-length(ord($1)))).ord($1)/ge;

      return '"' . $data . '"';
    } else {
      return $data;
    }
  }
}

sub printWords {
  my ($this,$data)= @_;
  return convertOutput($data,${*$this}{'translateStrings'});
}

sub strip
{
    my $line = shift;
    $line =~ s/^\s+//;
    $line =~ s/\s+$//;
    return $line;
}

sub getWords {
  my $this = shift;
  if(${*$this}{'errCode'} == $CGP::CLI_CODE_OK_INLINE) {
	return ${*$this}{'inlineResponse'};
  }
  my ($bag, $line) = ('', '');
  my $firstLine = $CGP::TRUE;
  my $lastLine = '';
  while ($CGP::TRUE) {
    $line = <$this>;
    chomp $line;
    $line = strip($line);
    if($firstLine) {
      $line =~ /^(.)/;
      if ($1) {
        $lastLine = '\)' if $1 eq '(';
        $lastLine = '\}' if $1 eq '{';
        $lastLine = $lastLine . '$';
        $firstLine = $CGP::FALSE;
      }
    }
    $bag .= $line;
    last if $line =~ /$lastLine/;
  }
  return $bag;
}


sub send {
  my ($this, $command) = @_;
  ${*$this}{currentCGateCommand} = $command;
  print STDERR ref($this) . "->send($command)\n\n"
    if ${*$this}{'debug'};
  ${*$this}{'lastAccess'}=time();
  print $this $command.$CGP::CLI_CRLF;
}


###########################

sub skipSpaces {
  my $this = shift;
  while(${*$this}{'span'} < ${*$this}{'len'} && substr(${*$this}{'data'},${*$this}{'span'},1) =~ /\s/) { ++${*$this}{'span'}; }
}

sub readWord {
  my $this = shift;
  my $isQuoted=0;
  my $isBlock=0;
  my $result="";

  $this->skipSpaces();
  if(substr(${*$this}{'data'},${*$this}{'span'},1) eq '"') {
    $isQuoted=1; ++${*$this}{'span'};
  } elsif(substr(${*$this}{'data'},${*$this}{'span'},1) eq '[') {
    $isBlock=1;
  }
  while(${*$this}{'span'} < ${*$this}{'len'}) {
    my $ch=substr(${*$this}{'data'},${*$this}{'span'},1);

    if($isQuoted) {
      if($ch eq '\\') {
        if(substr(${*$this}{'data'},${*$this}{'span'}+1,3) =~ /^(?:\"|\\|\d\d\d)/) {
          $ch=substr(${*$this}{'data'},++${*$this}{'span'},3);
          if($ch =~ /\d\d\d/) {
            ${*$this}{'span'}+=2;
            $ch=chr($ch);
          } else {
            $ch=substr($ch,0,1);
            $ch='\\'.$ch unless(${*$this}{'translateStrings'});
          }
        }
      } elsif($ch eq '"') {
        ++${*$this}{'span'};
        last;
      }
    } elsif($isBlock) {
      if($ch eq ']') {
        $result .= $ch;
        ++${*$this}{'span'};
        last;
      }
    } elsif($ch =~ /[-a-zA-Z0-9\x80-\xff_\.\@\!\#\%]/) {
    } else {
      last;
    }
    $result .= $ch;
    ++${*$this}{'span'};
  }
  return $result;
}


sub readKey() {
  my $this = shift;
  return $this->readWord();
}


sub readValue() {
  my $this = shift;
  $this->skipSpaces();
  my $ch=substr(${*$this}{'data'},${*$this}{'span'},1);
  if($ch eq '{') {
    ++${*$this}{'span'};
    return $this->readDictionary();
  } elsif($ch eq '(') {
    ++${*$this}{'span'};
    return $this->readArray();
  } else {
    return $this->readWord();
  }
}

sub readArray() {
  my $this = shift;
  my $result=[];
  while(${*$this}{'span'}<${*$this}{'len'}) {
    $this->skipSpaces();
    if(substr(${*$this}{'data'},${*$this}{'span'},1) eq ')') {
      ++${*$this}{'span'};
      last;
    } else {
      my $theValue=$this->readValue();
      $this->skipSpaces();
      push(@$result,$theValue);
      if(substr(${*$this}{'data'},${*$this}{'span'},1) eq ',') {
        ++${*$this}{'span'};
      } elsif(substr(${*$this}{'data'},${*$this}{'span'},1) eq ')') {
      } else {
        croak "CGPro output format error:",substr(${*$this}{'data'},${*$this}{'span'},10);
      }
    }
  }
  return $result;
}

sub readDictionary {
  my $this = shift;
  my $result={};
  while(${*$this}{'span'} < ${*$this}{'len'}) {
    $this->skipSpaces();
    if(substr(${*$this}{'data'},${*$this}{'span'},1) eq '}') {
      ++${*$this}{'span'};
      last;
    } else {
      my $theKey=$this->readKey();
      $this->skipSpaces();
      if(substr(${*$this}{'data'},${*$this}{'span'},1) ne '=') { croak "CGPro output format error:",substr(${*$this}{'data'},${*$this}{'span'},10); }
      ++${*$this}{'span'};
      @$result{$theKey}=$this->readValue();
      $this->skipSpaces();
      if(substr(${*$this}{'data'},${*$this}{'span'},1) ne ';') { croak "CGPro output format error:",substr(${*$this}{'data'},${*$this}{'span'},10); }
      ++${*$this}{'span'};
    }
  }
  return $result;
}

sub parseWords {
  my $this = shift;

  ${*$this}{'data'}=shift;
  ${*$this}{'span'}=0;
  ${*$this}{'len'}=length(${*$this}{'data'});
  return $this->readValue();
}

1;

package main;

use vars qw( %bin
             $dumpDir
             $ptrArgs
             $pwd
             $pleskVersion
             $pleskMajorVersion
             $pleskMinorVersion
             $pleskMinorMinorVersion
             $workDir);

use strict;

use File::Temp;

my $VERSION='1.0.1';

my $version;

my $kilobyte = 1024;
my $megabyte = $kilobyte*$kilobyte;
my $gigabyte = $megabyte*$kilobyte;

my $adminLogin;
my $adminPassword;

my $shadow = "migration/.shadow";
my $adminSettings = "/var/CommuniGate/Accounts/postmaster.macnt/account.settings";
my $globalSettings = "/var/CommuniGate/Settings/Main.settings";

#
# parse command line
#
my %arg_opts = ('--help|-h'=>'',
		'--config|-c'=>'s',
		'--debug|-d'=>'i?',
		'--get-status|-s'=>'',
		'--dump-accounts|-dc'=>'s',
		'--dump-domains|-dd'=>'s',
		'--dump-all|-da'=>'',
		'--get-content-list|-lc'=>'',
		'--no-content|-nc'=>'',
		'--output|-o'=>'s',
		'--status-file|-sf'=>'s',
		'--version|-v'=>'',
	       );

%bin = ('tar'=>'tar','split'=>'split','cp'=>'cp','rm'=>'rm','echo'=>'echo',
	   'mysql'=>'mysql','mysqldump'=>'mysqldump',
	   'psql'=>'psql','pg_dump'=>'pg_dump','uptime'=>'uptime','uname'=>'uname',
	   'cat'=>'cat','pwd'=>'pwd','md5sum'=>'md5sum',
	   'grep'=>'grep','find'=>'find','gzip'=>'gzip','wc'=>'wc',
	   'locale'=>'locale', 'iconv'=>'iconv');

my $returnCode = 0;



#==============================================================
#
# common subroutines
#

sub printToLog {
  my ($msg, $noeol) = @_;
  if ($msg) {
	print $msg;
  }
  unless ($noeol) {
	print "\n";
  }
}

sub printToError {
  my ($msg, $noeol, $errCode) = @_;
  if ($msg) {
	print STDERR $msg;
  }
  unless ($noeol) {
	print STDERR "\n";
  }
  if (defined($errCode)) {
    $returnCode = $errCode; ## translate an error code to the return code
  } else {
    $returnCode = 63;
  }
}

#BEGINNOTINRAQ2

sub makeFileReader
{
	my ($filename, $doNotFail) = @_;
	local (*INPUT);

	unless (-T $filename) {

		if (defined($doNotFail)) {
			&printToLog("Error: file '$filename' is not a text file");
		} else {
			&printToError("Error: file '$filename' is not a text file");
		}

		return undef;
	}

	unless (open(INPUT,"<$filename")) {

		binmode INPUT;
		if (defined($doNotFail)) {
			&printToLog("Error: unable to open file '$filename' to read");
		} else {
			&printToError("Error: unable to open file '$filename' to read");
		}

		return undef;
	}

	my $input = *INPUT;

	my $this = {
		'READ' => sub {
				my $a = <$input>;
				return $a;
			},

		'CLOSE' => sub { close($input); }

	};

	return $this;
}

sub makeArrayReader {
  my (@arr) = @_;
  my $size= 0;
  my $count = scalar(@arr);

  my $this={'READ'=>sub {
			  if($count == $size) {
				return undef;
			  }

			  return $arr[$count++];
			},
			'CLOSE'=>sub {
			}
		   };
  return $this;
}

sub makeParser {
  my $reader;
  my ($this,$ptrRows,$keySep,$valSep,$valKeySep,$comment,
      $typeRows,$maxIndex,$rowIndex);
  {
	$reader = shift;
	unless(defined($reader)) {
	  return undef;
	}

    my %args=(@_);
    $keySep = $args{'KEYSEPARATOR'};
    $valSep = $args{'VALUESEPARATOR'};
    $valKeySep = $args{'KEYVALUESEPARATOR'};
  }
  $this={'READER'=>sub {
		   if(@_){
			 $reader=shift;
		   }
		   return $reader;
		 },
		 'FILE'=>sub {
		   if(@_){
			 $reader = makeFileReader(@_);
		   }
		   return undef;
		 },
		 'KEYSEPARATOR'=>sub{
		   if(@_){
			 $keySep=shift;
		   }
		   return $keySep;
		 },
		 'VALUESEPARATOR'=>sub{
		   if(@_){
			 $valSep=shift;
		   }
		   return $valSep;
		 },
		 'KEYVALUESEPARATOR'=>sub{
		   if(@_){
			 $valKeySep=shift;
		   }
		   return $valKeySep;
		 },
		 'COMMENT'=>sub{
		   if(@_){
			 $comment=shift;
		   }
		   return $comment;
		 },
		 'PARSE'=>sub{
		   my %args=(@_);

		   $keySep=$args{'KEYSEPARATOR'} if exists $args{'KEYSEPARATOR'};
		   $valSep=$args{'VALUESEPARATOR'} if exists $args{'VALUESEPARATOR'};
		   $valKeySep=$args{'KEYVALUESEPARATOR'} if exists $args{'KEYVALUESEPARATOR'};
		   $comment=$args{'COMMENT'} if exists $args{'COMMENT'};

		   $ptrRows = &parseFile('reader'=>$reader,'keysep'=>$keySep,
								 'valsep'=>$valSep,'keyvalsep'=>$valKeySep,
								 'comment'=>$comment);
		   $typeRows = ref($ptrRows);
		   if($typeRows=~/ARRAY/){
			 $rowIndex=-1;
			 $maxIndex=scalar(@{$ptrRows})-1;
		   }
		   return $ptrRows;
		 },
		 'ROW'=>sub{
		   if($typeRows=~/HASH/){
			 my $key=shift;
			 return $ptrRows->{$key};
		   }elsif($typeRows=~/ARRAY/){
			 if($rowIndex<$maxIndex){
			   $rowIndex++;
			   return $ptrRows->[$rowIndex];
			 }else{
			   return undef;
			 }
		   }else{
			 return undef;
		   }
		 },
		};

  return $this;
}

sub makeFileParser {
  my $filename = shift;
  return makeParser(makeFileReader($filename), @_);
}

sub makeSafeFileParser {
  my $filename = shift;
  return makeParser(makeFileReader($filename, 'doNotFail'), @_);
}

sub makeStringParser {
  my $content = shift;
  return makeParser(makeArrayReader($content), @_);
}

sub parseFile {
  my %arg = (@_);
  my $reader = $arg{'reader'};
  my $keySep = $arg{'keysep'};
  my $valSep = $arg{'valsep'};
  my $valKeySep = $arg{'keyvalsep'};
  my $comment = $arg{'comment'};

  my ($ret,$key,$value,$reKey,$reValue,@values,$keyBlank,$valBlank,
     $vkey,$vvalue,$reKeyValue,$reComment);

  if($keySep){
    $ret = {};
    if(ref($keySep)=~/regexp/i || ($keySep eq ' ')){
      $reKey = $keySep;
    }else{
      $reKey = qr/$keySep/;
    }
  }else{
    $ret = [];
  }
  if($valSep){
    if(ref($valSep)=~/regexp/i || ($valSep eq ' ')){
      $reValue = $valSep;
    }else{
      $reValue = qr/$valSep/;
    }
  }
  if($valKeySep){
    if(ref($valKeySep)=~/regexp/i || ($valKeySep eq ' ')){
      $reKeyValue = $valKeySep;
    }else{
      $reKeyValue = qr/$valKeySep/;
    }
  }
  if($comment){
    if(ref($comment)=~/regexp/i){
      $reComment = $comment;
    }else{
      $reComment = qr/$comment/;
    }
  }

  while($_ = $reader->{'READ'}->()){
    chomp;
    next unless $_;
    if($comment && /$reComment/){  ## commentar
	  next;
    }
    if ($keySep){
      ($key,$value)=split($reKey,$_,2);
      if($key){
		$key=~s/^\s+//;
		$key=~s/\s+$//;
		if($valSep){

		  if ($valKeySep){
			$ret->{$key}={};
			foreach $value (split($reValue,$value)){
			  ($vkey,$vvalue)=split($reKeyValue,$value);
			  $vkey=~s/^\s+//;
			  $vkey=~s/\s+$//;
			  $ret->{$key}->{$value}=$vvalue;
			}
		  }else{
			push @{$ret->{$key}},split($reValue,$value);
		  }
		}else{
		  $value=~s/^\s+//;
		  $value=~s/\s+$//;
		  $ret->{$key}=$value;
		}
      }
    }else{
      if($valSep){
		push @{$ret},[split($valSep,$_)];
      }else{
		push @{$ret},$_;
      }
    }
  }

  $reader->{'CLOSE'}->();

  return $ret;
}

#ENDNOTINRAQ2

sub makePasswordNode {
  my $value= shift;
  my $type = shift||'encrypted';

  my $item = &makeXmlNode('password');
  $item->{'CONTENT'}->($value);  ## html encode
  $item->{'ATTRIBUTE'}->('type',$type);
  return $item;
}

sub mangleName {
  my $name = shift;

  if($name =~ /^\d/) {
    $name = 'user' . $name;
  }
  $name =~ s/\./_/g;

  return $name;
}

sub makeSysUserNode {
  my ($name, $password, $type) = @_;

  unless(defined($name) && defined($password) && defined($type)) {
	printToError("makeSysUserNode: Username, password or password type is not set.");
  }

  my $sysUserNode = makeXmlNode('sysuser');
  $sysUserNode->{'ATTRIBUTE'}->('name', mangleName($name));
  $sysUserNode->{'ADDCHILD'}->(makePasswordNode($password, $type));

  return $sysUserNode;
}

# Returns home directory for the system account $user
sub getHomeDir {
	my ($user) = @_;
	my $home;
	my @pw = getpwnam($user);
	if (-1 != $#pw) {
		$home = $pw[7];
	}
	unless ($home) {
		# how it is by default
		$home = '/home/$account';
	}
	return $home;
}

# Returns password for system account $user
sub getSystemPassword {
	my ($user) = @_;
	my $password;
	my @pw = getpwnam($user);
	if (-1 != $#pw) {
		$password = $pw[1];
	}
	unless ($password) {
		$password = '';
	}
	return $password;
}

sub createUserNameFromDomain {
  my ($name) = @_;

  if ($name =~ /^\d/) {
	$name = 'u' . $name;
  }

  $name =~ s/\./_/g;
  $name =~ s/^www\.//g;
  $name =~ s/[aoeui]//g if length($name) > 16;
  $name = substr($name, 0, 16) if length($name) > 16;

  return $name;
}

my $compress = 1;

sub setCompress {
  ($compress) = @_;
}

sub makeDumpDb
{
  my ($dbName, $dbType, $dbUser, $dbPasswd, $dbHost, $ptrDomainBin, $variables, $domainSocket) = @_;
  $dbType ||= 'mysql';

  my ($dumpFile,$dumpSql,$dumpName,$cmd,$gzip,$split,$mysql,$psql, $retCode,$out);

  $dumpName = "$dbName.$dbType.sql";
  $gzip = $compress == 1 ? ($bin{'gzip'} || 'gzip') : undef;

  if (-x $gzip) {
	$dumpName .= ".gz";
  }
  $dumpFile="$dumpDir/$dumpName";

  printToLog(">>Checking database presence");

  if ($dbType eq 'mysql') {
	$mysql = $bin{'mysql'} || 'mysql';
	if (-x $mysql) {
	  $cmd  = "$mysql -u '$dbUser'";
	  $cmd .= " -h $dbHost" if $dbHost;
	  $cmd .= " -p'$dbPasswd'" if $dbPasswd;
	  $cmd .= " -e '' '$dbName' 2>&1";

	  $out = `$cmd`;
	  chomp $out;
	  if ($out) {
		$retCode = $?>>8;
		printToLog("warning: mysql datatbase '$dbName' not found. mysql has returned '$retCode'");
		return "";
	  }
	}
  } elsif ($dbType eq 'Pg') {
	$psql = $bin{'psql'} || 'psql';
	if (-x $psql) {
	  $cmd  = "export PGUSER='$dbUser'";
	  $cmd .= " PGPASSWORD='$dbPasswd'" if $dbPasswd;
	  $cmd .= "; $psql '$dbName' ";
	  $cmd .= " -h $dbHost" if $dbHost;
	  $cmd .= " -c '' 2>&1";

	  $out = `$cmd`;
	  chomp $out;
	  if ($out) {
		$retCode = $?>>8;
		printToLog("warning: PostgreSQL datatbase '$dbName' not found. psql has returned '$retCode'");
		return "";
	  }
	}
  }

  printToLog("..done");

  unless (exists($ptrArgs->{'no-content'})) {
	$gzip = $bin{'gzip'} || 'gzip';
	$split = $bin{'split'} || 'split';

	if ($dbType eq 'mysql') {
	  my ($prefx, $postfix) = ("", "");
	  my $commonprefix = "LD_LIBRARY_PATH=$variables " if $variables;

	  my $commonopts;
	  $commonopts = " --socket='$domainSocket'" if $domainSocket;
	  $commonopts .= " -h $dbHost" if $dbHost;
	  $commonopts .= " -p'$dbPasswd'" if $dbPasswd;
	  $commonopts .= " -u '$dbUser'";

	  my $dumpcmd;
	  if ($ptrDomainBin) {
		$dumpcmd = $ptrDomainBin->{'mysqldump'};
	  } else {
		$dumpcmd = $bin{'mysqldump'} || 'mysqldump';
	  }

	  my $dumpopts = " --quick --quote-names --add-drop-table";

	  my $mysqlversion = `$commonprefix $bin{'mysql'} $commonopts --version`;
	  if ($mysqlversion =~ /Distrib (\d+)\.(\d+)\./) {
		if ($1 > 4 or ($1 == 4 and $2 > 0)) {
		  $dumpopts .= " --compatible=mysql323";
		}
	  }

	  $cmd = $commonprefix . $dumpcmd . $commonopts . $dumpopts . " '$dbName'";

	} elsif ($dbType eq 'Pg') {
	  $cmd  = "export PGUSER='$dbUser'";
	  $cmd .= " PGPASSWORD='$dbPasswd'" if($dbPasswd);
	  $cmd .= ' ; ' . ($bin{'pg_dump'} || 'pg_dump') . ' -d -O';
	  $cmd .= " -h '$dbHost'" if $dbHost;
	  $cmd .= " '$dbName'";
	}

	if ($compress and $gzip and -x $gzip) {
	  $cmd .= " | $gzip";
	}
	if (-x $split) {
	  $cmd .= " | $split -b 2040m - '$dumpFile.'";
	} else {
	  $cmd .= " > '$dumpFile'";
	}

	local $SIG{'PIPE'} = 'DEFAULT' if $SIG{'PIPE'} eq 'IGNORE';

	unless (system($cmd) == 0) {
	  printToError("Error execute '$cmd'");
	  return undef;
	}
  }
  return $dumpName;
}

sub addDbUsers {
  my ($dbNode, $dbName, $wrapUserMysql) = @_;

  printToLog("Dupming $dbName database users");

  $dbName =~ s/_/\\_/g;

  my (@users, $dbUser, $ptrRow);
  my $sql = "SELECT DISTINCT User FROM db WHERE Db = '$dbName'";
  $wrapUserMysql->{'EXECUTE'}->($sql);
  while ($ptrRow = $wrapUserMysql->{'FETCHROW'}->()) {
		push @users, (@{$ptrRow})[0];
  }
  $wrapUserMysql->{'FINISH'}->();

  foreach $dbUser (@users) {
		addDbUser($dbNode, $dbUser, $wrapUserMysql);
  }
}

sub addDbUser {
  my ($dbNode, $user, $wrapUserMysql, $passwordNode, $access_hosts) = @_;

  &printToLog("Dumping mysql user '$user'");

	my ( $item );

  if ( ref( $wrapUserMysql ) =~ /HASH/ &&
			 ref( $wrapUserMysql->{'EXECUTE'} ) =~ /CODE/ ) {

    my ($sql,$ptrRow,@accesshosts,$accesshost);

    $sql = "SELECT password,host FROM user WHERE user='$user'";

    if ( $wrapUserMysql->{'EXECUTE'}->( $sql ) ) {

			$item = makeXmlNode( 'dbuser' );
			$item->{'ATTRIBUTE'}->('name', $user);

      while ( $ptrRow = $wrapUserMysql->{'FETCHROW'}->() ) {
				unless ( ref($passwordNode) =~ /HASH/ ) {
					$passwordNode = makePasswordNode( $ptrRow->[0] );
				}
				push @accesshosts, $ptrRow->[1];
      }

			if ( $passwordNode ) {
				$item->{'ADDCHILD'}->( $passwordNode );
			}

			if (defined($access_hosts) && ref($access_hosts)=~/ARRAY/) {
				@accesshosts = @{$access_hosts};
			}

			foreach $accesshost ( @accesshosts ) {
				$item->{'ADDCHILD'}->( makeXmlNode( 'accesshost', $accesshost ) );
			}

			$dbNode->{'ADDCHILD'}->( $item );

    }

    $wrapUserMysql->{'FINISH'}->();
  }

	return $item;
}

my $lastDumpId = 0;

sub generateDumpId {
  return $$ . "-" . $lastDumpId++;
}

#
# sub makeDumpFile - make a dump of dir tree (use 'tar')
#
# Arguments:
#          $destFile - name of destination file.
#                      If the file has not '.tgz' or '.tar.gz' extension
#                      '.tgz' will be added
#          $scrDir   - path to source dir
#
#          $ptrNames - pointer to an array with names of files and dirs to
#                      include into dump
#                      can be a single name
#                      by default '.' - include all entries
#
#          $ptrExcludes - pointer to an array with names of files and dirs to
#                      exclude from dump
#                      can be a single name
#
#          $extraParams - extra params to run tar
#                         ('-h' - don't dump symlinks; dump the files they point to)
#
sub makeDumpFile {
  my ($destFile, $srcDir, $ptrNames,$ptrExclude,$extraParams)=@_;

  my @destPath = split(/\//, $destFile);
  if (length($destPath[$#destPath]) > 32) {
	$destPath[$#destPath] = generateDumpId();
  }
  $destFile = join('/', @destPath);

  my $tar = $bin{'tar'} || $bin{'bsdtar'} || die "unable to find tar";

  my $split = $bin{'split'}||'split';
  my $gzip = $compress == 1 ? ($bin{'gzip'} || 'gzip') : undef;

  my ($cmd,$ret,@names,@exclude);

  if(ref($ptrNames)=~/ARRAY/){
    @names = @{$ptrNames};
  }else{
    @names=($ptrNames) if $ptrNames;
  }

  if(ref($ptrExclude)=~/ARRAY/){
    @exclude = @{$ptrExclude};
  }else{
    @exclude=($ptrExclude) if $ptrExclude;
  }

  unless(-x $tar ){
    &printToError("Error: '$tar' is not executable");
    return undef;
  }

#
# check extension
#
  if(-x $gzip){
    unless($destFile=~/\.tgz$/ ||$destFile=~/\.tar\.gz$/){
      $destFile .= '.tar.gz';
    }
  }else{
    unless($destFile=~/\.tar$/){
      $destFile .= '.tar';
    }
  }
#
# end check extension
#

  unless( exists $ptrArgs->{'no-content'}){

    $cmd=$tar;
    if($srcDir){
      if( -d $srcDir){
		$cmd.=" -C $srcDir -f - -c ";
		if ($bin{'tar'}) { # GNU tar present -> understand --ignore-failed-read
		  $cmd .= " --ignore-failed-read ";
		}
      }else{
		return undef;
      }
    }


    if($extraParams){
      $cmd .= $extraParams.' ';
    }

    if(@exclude){
      $cmd .= ' '.join( ' ', map {s/^\Q$srcDir\E\///;'--exclude '.$_} @exclude); ## extract domain name
    }

	# stop treating arguments as options, if tar is recent enough
	if ($bin{'tar-accepts-doubledash'}) {
	  $cmd .= ' -- ';
	}

    if(@names){
	  @names = map { "'$_'" } @names;
      if($srcDir){
		$cmd .= join( ' ', map {s/^\Q$srcDir\E\///;$_} @names); ## extract domain name
      }else{
		$cmd .= join( ' ', @names );
      }
    }else{
      $cmd .= '.';  ## dump all files & dirs
    }

    if(-x $gzip){
      $cmd .= " | $gzip";
    }
    if(-x $split){
      $cmd .= " | $split -b 2040m - $destFile.";
    }else{
      $cmd .= " >$destFile";
    }

		local $SIG{'PIPE'} = 'DEFAULT' if $SIG{'PIPE'} eq 'IGNORE';

    unless(system($cmd) == 0){
      &printToError("Error execute command: '$cmd': $!");
      $destFile='';
    }
  }

  $destFile=~s/^\Q$dumpDir\E\///;  ### remove dir name
  return $destFile;
}

sub getDumpDir {
  my ($subdir,$clean) = @_;
  my ($dumpdir);

  if($subdir=~/^\//){
    $dumpdir=$subdir;
  }else{
    my $util = $bin{'pwd'}||'pwd';
    $dumpdir = $pwd||`$util`;
    chomp $dumpdir;
    if($subdir){
      $dumpdir.='/'.$subdir;
    }
  }
  if(-d $dumpdir && $clean){
    unless(system("rm -rf $dumpdir")==0){
      &printToError("Error: can not clean temporary dir '$dumpdir': $!");
    }
  }
  unless(-d $dumpdir){
    mkdir ($dumpdir,0755) or
      die "Error: can not create temporary dir '$dumpdir': $!\n";
  }
  return $dumpdir;
}

sub printXml {
  my ($root,$outFh,$dtd) = @_;
  unless($outFh){
    $outFh = *STDOUT;
  }

  unless(ref($root)=~/HASH/ && ref($root->{'PRINT'})=~/CODE/){
    print $outFh "Error: a xml root is not found\n";
    &printToError("Error: sub printXml: bad arguments");
    return 0;
  }
  print $outFh "<?xml version='1.0' encoding=\"UTF-8\"?>\n";

  my  $doctype = $root->{'NAME'}->();
  if($dtd){
    print $outFh "<!DOCTYPE $doctype SYSTEM '$dtd'>\n";
  }
  $root->{'PRINT'}->($outFh,'  ');
  return 1;
}

#
# XmlNode("blah", "content" => "blah",
#                 "children" => [$foo, $bar],
#                 "attributes" => {"botva"=>"7", "foo"=>"bar"})
#
sub XmlNode {
  my ($name, %args) = @_;

  my $node;
  if ($args{'content'}) {
	$node = makeXmlNode($name, $args{'content'});
  } else {
	$node = makeXmlNode($name);
  }

  if ($args{'attributes'}) {
	foreach my $attribute (keys %{$args{'attributes'}}) {
	  $node->{'ATTRIBUTE'}->($attribute, $args{'attributes'}->{$attribute});
	}
  }

  if ($args{'children'}) {
	foreach my $child (@{$args{'children'}}) {
	  $node->{'ADDCHILD'}->($child);
	}
  }
  return $node;
}

sub makeXmlNode {
  my ($this,$parent,%attributes,@content);
  my ($name,$plain)=@_;

  $plain = toUtf8(encodeHtml($plain));

  unless($name||$plain){
    return undef;
  }
  $this = {
	   'ATTRIBUTE' => sub {
	     my $key = shift;
	     if(@_){
	       my $value=shift;
	       if(@_){
					 $attributes{$key} = $value;
	       }else{
					 $attributes{$key} = toUtf8(normalizeAttribute($value));
	       }
	     }
	     return $attributes{$key};
	   },
	   'PARENT' => sub {
	     if(@_){
	       my $arg = shift;
	       if(ref($arg)=~/HASH/){
					 $parent=shift;
	       }
	     }
	     return $parent;
	   },
	   'NAME' => sub {
	     if(@_){
	       $name = shift;
	     }
	     return $name;
	   },
	   'CONTENT' => sub {
	     if(@_){
	       $plain = toUtf8(encodeHtml(shift));
	     }
	     return $plain;
	   },
	   'ADDCHILD'=> sub {
	     my $kind = shift;
	     if(
					(ref($kind)=~/HASH/) &&
					(ref($kind->{'PARENT'})=~/CODE/)
	       ) {
	       push @content,$kind;
	       $kind->{'PARENT'}->($this);
	     }
	     return $#content;
	   },
	   'KIND'=> sub {
	     my $index = shift;
			 my ($ret);
			 if ( $index =~ /^\d+$/ ) {
				 if($index >= 0 && $index <= $#content ) {
					 $ret =  $content[$index];
				 }
			 } else {
				 foreach my $item (@content){
					 if ( $item->{'NAME'}->() eq $index ) {
						 $ret = $item;
						 last;
					 }
				 }
			 }
			 return $ret;
	   },
	   'KINDSREF'=> sub {
			 my $name = shift;
			 my $ret = \@content;
			 if ( $name ) {
				 $ret = [];
				 foreach my $item (@content) {
					 if ( $item->{'NAME'}->() eq $name ) {
						 push @{$ret}, $item;
					 }
				 }
			 }
	     return $ret;
	   },
	   'ATTRIBUTESREF' => sub {
	     return \%attributes;
	   },
	   'PRINT'=>sub {
	     my $fh = shift || *STDOUT;
	     my $addPrefix = shift || '';
	     my $prefix = shift || '';
	     if($name){
	       my ($out,$item);
	       if(@content || defined($plain)){

					 $out = $prefix . &getTag($name,\%attributes,'start');
					 print $fh $out;
					 if(@content){
						 print $fh "\n";
						 foreach $item (@content){
							 if(ref($item)=~/HASH/ && ref($item->{'PRINT'})=~/CODE/){
								 $item->{'PRINT'}->($fh,$addPrefix,$prefix.$addPrefix);
							 }
						 }
					 }
					 if(defined($plain)){
						 print $fh $plain;
						 if(@content){
							 print $fh "\n$prefix";
						 }
					 }else{
						 if($prefix){
							 print $fh $prefix;
						 }
					 }
					 $out = &getTag($name,\%attributes,'stop') . "\n";
					 print $fh $out;

	       }else{
					 $out = $prefix . &getTag($name,\%attributes) . "\n";
					 print $fh $out;
	       }
	     }else{
	       if(defined($plain)){
					 print $fh $plain;
	       }
	     }
	   }
	 };

  return $this;
}

sub getTag {
  my ($name,$ptrAttr,$mode) = @_;
  my ($out,$key,$value);
  $mode||='';
  if ($mode eq 'stop'){
    $out="</$name>";
  }else{
    $out="<$name";
    while (($key,$value)=each(%{$ptrAttr})){
      $out .= ' '.$key;
      if (defined($value)){
		$out .= '="'.$value.'"';
      }
    }
    if($mode eq 'start'){
      $out .= ">";
    }else{
      $out .= "/>";
    }
  }
  return $out;
}

#
# end class for handle with XML
#


#
# class for handle with database
#
sub getDbConnect {
  my $ptrBin = shift;
  my $myDbType = shift;
  my $myDbAddress = shift ;
  my $myDbUser = shift ;
  my $myDbPasswd = shift ;
  my $myDbDB = shift ;
  my $myDbHost = shift ;
  my $continueAfterError = shift;

  #added by Honig for Sphera
  my $myDbSocket = shift;
  my $variables = shift;

  my ($dbh,$this,$sth);
  my ($shell,$args);
  my $echo = $ptrBin->{'echo'}||'echo';
  my ($sql,@result,$resultIndex,$ptrResult);

  unless(defined($myDbPasswd) && defined($myDbUser)){
    if($myDbType eq 'mysql'){
      foreach my $myCnf ('/etc/my.cnf',$ENV{'HOME'}.'/.my.cnf'){
		next unless (-T $myCnf);
		next unless open(CNF,"<$myCnf");
		binmode CNF;
		my($key,$value);
		my $state=0;
		while (<CNF>){
		  next unless $_;
		  next if (/^#/);
		  if($state){
			if(/^\s*\[/){
			  last;
			}
			if(/^\s*(\S+)\s*=\s*(\S+)/){
			  $key=$1;
			  $value=$2;
			  if($key =~ /^user$/){
				$myDbUser = $value;
				# sometimes strings are quoted
				$myDbUser =~ s/^('|")(.*)\1$/$2/;
			  }elsif($key =~ /^pass$/){
				$myDbPasswd = $value;
				# sometimes strings are quoted
				$myDbPasswd =~ s/^('|")(.*)\1$/$2/;
			  }
			}
		  }else{
			if(/^\s*\[client\]/){
			  $state=1;
			}
		  }
		}
		close (CNF);
      }
    }elsif($myDbType eq 'Pg'){
      $myDbPasswd ||= $ENV{'PGPASSWORD'};
      $myDbUser ||= $ENV{'PGUSER'};
    }
  }

  if (eval "require DBI") {
    if (eval "require DBD::$myDbType"){
      if($myDbAddress){
		if($myDbUser && $myDbPasswd){
		  unless ($dbh = DBI->connect($myDbAddress,$myDbUser,$myDbPasswd)) {
			return if $continueAfterError;
			die "Error: unable to connect to database: $DBI::errstr\n";
		  }
		}elsif($myDbUser){
		  unless ($dbh = DBI->connect($myDbAddress,$myDbUser)) {
			return if $continueAfterError;
			die "Error: unable to connect to database: $DBI::errstr\n";
		  }
		}else{
		  unless ($dbh = DBI->connect($myDbAddress)) {
			return if $continueAfterError;
			die "Error: unable to connect to database: $DBI::errstr\n";
		  }
		}
      }
    }
  }

#  unless($dbh){
#    if ($myDbType eq 'Pg'){
#      if(eval "require Pg"){
#        $myDbAddress="user=$myDbUser dbname=$myDbDB";
#        if($myDbPasswd){
#          $myDbAddress.=" password=$myDbPasswd";
#        }
#        $dbh = Pg::connectdb($myDbAddress);
#      }
#    }
# }
	unless ($dbh) {

		if ($myDbType eq 'mysql') {
			$shell = $ptrBin->{'mysql'} || 'mysql';
			$args  = " --socket='$myDbSocket'" if ($myDbSocket);
			if ($myDbPasswd) {
				$args .= " -B -u '$myDbUser' -p'$myDbPasswd' '$myDbDB' ";
			} else {
				$args .= " -B -u $myDbUser $myDbDB ";
			}
			$shell = "LD_LIBRARY_PATH=$variables $shell" if ($variables);
    }elsif($myDbType eq 'Pg'){
      $shell = $ptrBin->{'psql'}||'psql';
	  if($myDbPasswd) {
		$shell = "PGPASSWORD=$myDbPasswd $shell";
	  }
	  if($myDbUser) {
		$shell = "PGUSER=$myDbUser $shell";
	  }

      $args = " -d $myDbDB -c ";
    }
#
# check connection to Confixx databnase
#
    if($myDbDB=~/confixx/){
      $sql = "select hostname from admin";
      @result = `$echo "$sql"| $shell $args`;
      unless($result[0]=~/^\w+$/){
		$args = ' -h $myDbHost'.$args;
		@result = `$echo "$sql"| $shell $args`;
		unless($result[0]=~/^\s*hostname/){
		  die "Error: Can not connect to Confixx database (unable to execute sql: $sql)\n@result\n";
		}
      }
    }
#
# end check connection to Confixx databnase
#
    $shell .= $args;
  }

  $this = {
		'RECONNECT'=>sub{
			if($dbh){
				$dbh->disconnect();
				if($myDbPasswd){
					$dbh = DBI->connect($myDbAddress,$myDbUser,$myDbPasswd) or
						die "Error: unable to connect to database: $DBI::errstr\n";
				}else{
					$dbh = DBI->connect($myDbAddress,$myDbUser) or
						die "Error: unable to connect to database: $DBI::errstr\n";
				}
				return $dbh;
			}else{
				return 1;
			}
		},
	   'EXECUTE_INT' => sub {
	     my $sql = shift;
		 my $already_reconnected = shift;
	     if ($dbh) {
	       $sth = executeDbh($dbh,$sql);
	       if ($sth) {
			 return $sth->rows;
	       } else {
			 # Wired MySQL timeouts on some DBI installations even on actively used connections
			 if ($DBI::err == 2006 && !$already_reconnected) {
				printToError("MySQL connection timeout detected - attempting to reconnect... ", 1);
				unless ($this->{'RECONNECT'}->()) {
					printToError("FAILED");
					return 0;
				}
				printToError("OK");
				return $this->{'EXECUTE_INT'}->($sql, 1);
			 }
			 return 0;
	       }
	     } else {
	       $resultIndex = 0;
	       executeDbShell($shell, \@result, $echo, $sql, $myDbType);
	       return scalar(@result);
	     }
	   },
	   'EXECUTE' => sub {
		 if ($sql ne "") {
		   print "---> Unfinished statement $sql\n";
		 }
	     $sql = shift;
		 my $res = $this->{'EXECUTE_INT'}->($sql);
		 unless ($res) {
		   $sql = "";
		 }
		 return $res;
	   },
	   'FETCHROW'=> sub {
	     if($sth){
	       if(@result = $sth->fetchrow){
					 return \@result;
	       }else{
					 return undef;
	       }
	     }else{
	       $resultIndex++;
	       if($resultIndex>$#result){
					 return undef;
	       }else{
					 return $result[$resultIndex];
	       }
	     }
	   },
	   'FETCHHASH'=>sub {
	     if($sth){
	       $ptrResult = $sth->fetchrow_hashref();
	     }else{
	       $resultIndex++;
	       if($resultIndex>$#result){
					 $ptrResult = undef;
	       }else{
					 my ($i,$key);
					 $ptrResult = {};
					
					 for($i=0; $i<@{$result[0]}; $i++){
						 $ptrResult->{$result[0]->[$i]}=$result[$resultIndex]->[$i];
					 }
	       }
	     }
	     return $ptrResult;
	   },
		 'FIELDS' => sub { ## get list of field names
			 if($sth){
				 my @fields = ();
				 for(my $i=0; $i<$sth->{'NUM_OF_FIELDS'}; $i++){
					 push @fields, $sth->{'NAME'}->[$i];
				 }
				 return @fields;
			 }else{
				 return @{$result[0]};
			 }					
		 },
	   'FINISH'=>sub {
	     if($sth){
	       $sth->finish();
	     }
		 $sql = "";
	     @result=();
	     $ptrResult = undef;
	   },
	   'PRINTERROR'=>sub {
	     if($dbh){
	       if(@_){
			 $dbh->{'PrintError'}=shift;
	       }
	       return $dbh->{'PrintError'};
	     }else{
	       return 0;
	     }
	   },
	   'PING'=>sub{
	     if($dbh){
	       return $dbh->ping();
	     }else{
	       return 1;
	     }
	   },
	   'DISCONNECT'=>sub{
	     if($dbh){
	       return $dbh->disconnect();
	     }else{
	       return 1;
	     }
	   },
	   'USER'=>sub{
	     return $myDbUser;
	   },
	   'PASSWORD'=>sub{
	     return $myDbPasswd;
	   },
	   'HOST'=>sub{
	     return $myDbHost;
	   },
	   'TYPE'=>sub{
	     return $myDbType;
	   },
	  };

  return $this;
}

sub executeDbh {
  my $dbh = shift;
  unless(ref($dbh)=~/DBI/){
    &printToError ("Error: DBI object is not found");
  }
  my $sql = shift;

  my $sth = $dbh->prepare($sql) or
    &printToError ("Error: prepare sql: $sql\n'" . $DBI::errstr. "'", 0);
  if($sth->execute){
    return $sth;
  }else{
    &printToError ("Error: execute sql: $sql\n'" . $DBI::errstr."'", 0);
  }
  return undef;
}

sub executeDbShell {
  my ($shell,$ptrRes,$echo,$sql, $type) = @_;
  @{$ptrRes}=();
  my (@out,@row,$ret);
  $ret=0;
  if($type eq 'Pg'){
    @out = `$shell "$sql"`;
  }else{
	# backticks are used to quote `ssl` for MySQL 4
	$sql =~ s/`/\\`/g;
    @out = `$echo "$sql"|$shell`;
  }
  if ($out[0]=~/^error/i){
    &printToErrror ("Error: execute sql: $sql\n@out");

  }else{

    if($type eq 'Pg'){
      foreach my $line (@out){
				chomp $line;
				next if ($line=~/^-[-+]*$/); # skip delimiter
				if($line=~/^\(\s*\d+\s+\w+\)$/){
					last;  ## (6 rows)
				}
				$line=~s/^\s+//;
				$line=~s/\s+$//;
				@row = split(/\s*\|\s*/,$line);
				push @{$ptrRes}, [@row];
				$ret++;
			}

    }else{
      if($out[0]=~/^\+[-+]*$/){
				foreach my $line (@out){
					chomp $line;
					next if ($line=~/^\+[-+]*$/); # skip delimiter
					$line=~s/^\|\s*//;
					$line=~s/\s+$//;
					@row = split(/\s*\|\s*/,$line);
					@row = map { $_ eq 'NULL' ? undef : $_ } @row;
					push @{$ptrRes}, [@row];
					$ret++;
				}
      }else{  ## batch mode
				foreach my $line (@out){
					chomp $line;
					@row = split(/\t/,$line);
					@row = map { $_ eq 'NULL' ? undef : $_ } @row;
					push @{$ptrRes}, [@row];
					$ret++;
				}
      }
    }
  }
  return $ret;
}

#
# end class for handle with database
#

sub makeMIMEBase64 {
  my ($useModule,$this);
  if (eval "require MIME::Base64"){
    $useModule = 1;
  }else{
    $useModule = 0;
  }

  $this={
	 'ENCODE'=>sub{
	   my $src = shift;
	   my ($encoded);
	   if($useModule){
	     $encoded = MIME::Base64::encode($src);
		 chomp $encoded;
	   }else{
	     $encoded = &encodeBase64($src);
	   }
	   return $encoded;
	 },
	 'USEMODULE'=>sub{
	   return $useModule;
	 },
	};

  return $this;
}

#
# url decode (certificates in Plesk DB encoded by the url encoding)
#

sub urlDecode {
    my $url = $_[0];
    $url =~ tr/+/ /;
    $url =~ s/%([a-fA-F0-9]{2,2})/chr(hex($1))/eg;
    return $url;
}

sub encodeHtml {
  my(@ret,$src);
  if ($src=shift){
    if(ref $src){
      if(${$src}=~/[&"'><]/){
		${$src}=~s/&/&amp;/sg;
		${$src}=~s/>/&gt;/sg;
		${$src}=~s/</&lt;/sg;
		${$src}=~s/\'/&#39;/sg;
		${$src}=~s/\"/&quot;/sg;
      }
      return ${$src};
    }else{
      if($src=~/[&"'><]/){
		$src=~s/&/&amp;/sg;
		$src=~s/>/&gt;/sg;
		$src=~s/</&lt;/sg;
		$src=~s/\'/&#39;/sg;
		$src=~s/\"/&quot;/sg;
      }
    }
  }
  return $src;
}

# Normalization according to the XML 1.0 (Third Edition),
# clauses 2.3 "Common Syntactic Constructs"
# and 3.3.3 "Attribute-value normalization"

sub normalizeAttribute {
  $_ = $_[0];
  s/\&/&amp;/g;
  s/\r/&#xD;/g;
  s/\n/&#xA;/g;
  s/\t/&#x9;/g;
  s/\</&lt;/g;
  s/\"/&quot;/g;
  s/\'/&#39;/g;
  return $_;
}

############################
#
#	Non-ascii characters handling
{
	# static
	my $charset;

	sub setCharset {
	  $charset = shift;
	}

	# try to determine the local charmap
	sub _determineCharset()
	{
	  my $charset;
	  my $locale = $bin{'locale'} || 'locale';

	  if (exists $ENV{'LANG'}) {
		my @pieces = split /\./, $ENV{'LANG'};
		if (defined $pieces[1]) {
		  $charset = $pieces[1];
		  chomp $charset;
		}
	  }
	  if (not $charset && open(CM, "$locale charmap |")) {
	  	binmode CM;
		$charset = <CM>;
		close(CM);
		chomp $charset;
	  }
	  if (not $charset) {
		$charset = 'ISO-8859-1';
	  }

	  return $charset;
	}

	sub toUtf8
	{
		my ($subj, $myCharset) = @_;

		$charset = _determineCharset() if (not $charset);

		$myCharset = $charset unless ( $myCharset );

		if ( (not $subj) ||
				 (not $bin{'iconv'}) ||
				 ( $myCharset =~ /UTF-?8/i )
			 ) {
			return $subj;
		}

		if ($subj =~ /^[\x00-\x7f]*$/) { return $subj; }

		my $converted;
		my $escaped = $subj;
		$escaped =~ s/'/'"'"'/g;

		if (open(IC, "echo '$escaped' | " . $bin{'iconv'} . " -f '$myCharset' -t 'UTF-8' |")) {
			binmode IC;
			while (my $string = <IC>) { $converted .= $string; }
			close(IC);
			chomp $converted;
			if ($converted) {
			  return $converted;
			}
		}

		return "";
	}
}

#
# Based on the b64.c by Bob Trower (base64.sf.net)
#

my @cb64 = ('A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
			'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
			'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
			'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
			'g', 'h', 'i', 'j',	'k', 'l', 'm', 'n',
			'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
			'w', 'x', 'y', 'z', '0', '1', '2', '3',
			'4', '5', '6', '7', '8', '9', '+', '/');

@cb64 = map { ord } @cb64;

sub encodeBlock {
  my ($len, @in) = @_;
  my @res;
  push @res, $cb64[$in[0] >> 2];
  push @res, $cb64[(($in[0] & 0x3)<<4)|(($in[1]&0xf0)>>4)];
  push @res, $len>1 ? $cb64[(($in[1]&0xf)<<2)|(($in[2]&0xc0)>>6)] : ord('=');
  push @res, $len>2 ? $cb64[$in[2]&0x3f] : ord('=');
  return @res;
}

sub encodeBase64 {
  my @in = unpack "C*", $_[0];

  my @out;
  while (scalar(@in)) {
	my @block = (0, 0, 0);
	my $length = 0;
	while (scalar(@in) && $length < 3) {
	  $block[$length++] = shift(@in);
	}
	push @out, encodeBlock($length, @block);
  }
  return pack "C*", @out;
}

sub resolveFile {
  my ($prefixes, $file) = @_;

  foreach my $prefix (@{$prefixes}) {
	$prefix.='/' unless $prefix=~/\/$/;
	$file =~ s|^/||;

	return ($prefix.$file) if (-x ($prefix.$file));
  }
}

sub checkGnuTar {
  my ($file) = @_;

  # on some FreeBSD this command outputs on stderr
  open (TAR, "$file --version 2>&1 |");
  while (<TAR>) {
	return 1 if (/GNU tar/);
  }
  return 0;
}

sub getBins {
  my ($ptrBin,$path) = @_;
  my (%searchPath,$key,$prefix,$bin,$fullPath);
  my ($notFound,$i,@prefixes);

  $notFound=0;

  $i=0;
  foreach $prefix (split(/:/,$path)){
    next unless $prefix;
    unless(exists($searchPath{$prefix})){
      $searchPath{$prefix}=$i++;
      push @prefixes,$prefix;
    }
  }

  foreach $key (keys %{$ptrBin}){
	$ptrBin->{$key} = resolveFile(\@prefixes, $ptrBin->{$key});

	# special case: 'tar' *must* be GNU tar

	if ($key eq 'tar') {
	  my $foundGnuTar = checkGnuTar($ptrBin->{'tar'});

	  # if 'tar' is not GNU tar, try 'gtar'
	  if (!$foundGnuTar) {
		if (checkGnuTar(resolveFile(\@prefixes, 'gtar'))) {
		  $ptrBin->{'tar'} = resolveFile(\@prefixes, 'gtar');
		} else {
		  $ptrBin->{'tar'} = undef;
		}
	  }

	  if ($ptrBin->{'tar'}) {
		open (TAR, "$ptrBin->{'tar'} --version|");
		while (<TAR>) {
		  if (/tar \(GNU tar\) (\d+)\.(\d+)/) {
			if ($1 > 1 or ($1 == 1 and $2 >= 13)) {
			  $ptrBin->{'tar-accepts-doubledash'} = 1;
			}
			last;
		  }
		}
		close (TAR);
	  }
	}

	# bsdtar case: it is fallback case and should be set up
	# 1) to 'tar' if 'tar' is not GNU tar
	# 2) to 'bsdtar' if no 'tar' found in system

	if ($key eq 'bsdtar') {
	  # if 'bsdtar' not found, but 'tar' exists and not a GNU tar
	  unless ($ptrBin->{'bsdtar'}) {
		if (resolveFile(\@prefixes, 'tar')) {
		  unless (checkGnuTar(resolveFile(\@prefixes, 'tar'))) {
			$ptrBin->{'bsdtar'} = resolveFile(\@prefixes, 'bsdtar');
		  }
		}
	  }
	}

    $notFound++;
  }

  return $notFound;
}

sub getArguments {
  my ($ptrArgv,$ptrOpts) = @_;

  my (@keys,$firstKey,%prepKeys,$key,$value,$ptrArr,$arg,$state);
  my (%retHash,$pat,$found,$used,@rest,$fullArg,$prevKey);

  while (($key,$value)=each(%{$ptrOpts})){
    @keys = split(/\|/,$key);
    $firstKey = $keys[0];
    $firstKey =~s/^-*//;
    push @{$prepKeys{$firstKey}},[@keys];
    push @{$prepKeys{$firstKey}},$value;
  }

  $state =0;
  foreach $arg (@{$ptrArgv}){

#    print "arg: $arg\n";

    $used = 0;
    if($state==1){
      if  ($arg =~ /^-/){
	$state=0;
      }else{
	$used = 1;
	$retHash{$prevKey}=$arg;
	next;
      }
    }
    if ($state == 2){
      $retHash{$prevKey}=$arg;
      $state = 0;
      $used = 1;
    }else{
      $fullArg = $arg;
      if ($arg =~ /^(-\S+)=(\S+)/){
	$arg = $1;
	$value = $2;
      }else{
	$value = undef;
      }
      foreach $key (keys %prepKeys){

	$ptrArr = $prepKeys{$key};

#	print "key: $key, ",join('|',@{$ptrArr->[0]}),"\n";


	$found = 0;
	foreach $pat (@{$ptrArr->[0]}){
	  if ($pat eq $arg){
	    $found = 1;
	    last;
	  }
	}
	if($found){
	  $used = 1;
	  if(defined($value)){
	    $retHash{$key}=$value;
	  }else{
	    if($ptrArr->[1]){
	      if($ptrArr->[1] =~ /\?$/){
		$state = 1;
	      }else{
		$state = 2;
	      }
	      $prevKey = $key;
	    }else{
	      $retHash{$key}=undef;
	    }
	  }
	  last;
	}
      }
    }
    unless($used){
      if($value){
	push @rest,$fullArg;
      }else{
	push @rest,$arg;
      }
    }
  }
  @{$ptrArgv}=@rest;
  return \%retHash;

}

sub makeDumpStatus($) {
  my $statusFile = shift;
  my($pid,$currAccount,$currDomain,$leftAccounts,$leftDomains,$this);
  $pid=$$;
  $this = {'FILE'=>sub {
	     if(@_){
	       $statusFile = shift;
	     }
	     return $statusFile;
	   },
	   'PID'=>sub {
	     if(@_){
	       $pid = shift;
	     }
	     return $pid;
	   },
	   'ACCOUNT'=>sub{
	     if(@_){
	       $currAccount = shift;
	       if($leftAccounts){
		 $leftAccounts--;
	       }
	     }
	     return $currAccount;
	   },
	   'DOMAIN'=>sub{
	     if(@_){
	       $currDomain = shift;
	       if($leftDomains){
		 $leftDomains--;
	       }
	     }
	     return $currDomain;
	   },
	   'COUNTS'=>sub{
	     if(@_){
	       my $newValue = shift;
	       if (defined $newValue){
		 $leftAccounts=$newValue;
	       }
	       $newValue = shift;
	       if (defined $newValue){
		 $leftDomains=$newValue;
	       }
	     }
	     return ($leftAccounts,$leftDomains);
	   },
	   'PRINT'=>sub{
	     if(@_){
	       $statusFile=shift;
	     }
	     open(STATUS,">$statusFile.tmp");

		 binmode STATUS;
	     my $root = &makeXmlNode('agent-dump-status');
	     my $item = &makeXmlNode('pid',$pid);
	     $root->{'ADDCHILD'}->($item);
	     $item = &makeXmlNode('account',$currAccount||'');
	     $root->{'ADDCHILD'}->($item);
	     $item = &makeXmlNode('domain',$currDomain||'');
	     $root->{'ADDCHILD'}->($item);
	     $item = &makeXmlNode('accounts-left',
				  defined($leftAccounts)?$leftAccounts:'');
	     $root->{'ADDCHILD'}->($item);
	     $item = &makeXmlNode('domains-left',
				  defined($leftDomains)?$leftDomains:'');
	     $root->{'ADDCHILD'}->($item);

	     &printXml($root,*STATUS,'agent-dump-status.dtd');

	     close(STATUS);

		 rename("$statusFile.tmp", $statusFile);
	   }

	  };
  return $this;
}

sub makeContentList {
  my ($dirDump,$xmlDump) = @_;
  my ($name,$fullPath,%files,%fromXML,$size,@cids,$root,$item,$key,$cmd,
      $gzip,$wc,$md5sum);

  if(opendir(DUMP,$dirDump)){
    while ($name=readdir(DUMP)){
      $fullPath="$dirDump/$name";
      unless(-f $fullPath && $name=~/\.\w\w$/){
	next;
      }
      $key = $fullPath;
      $key =~ s/\.\w\w$//;
      $size=(stat($fullPath))[7];
      if(defined $size){
	push @{$files{$key}},[$fullPath,$size];
      }
    }
    closedir(DUMP);
  }

	if (-T $xmlDump) {
		if (open(DUMP, "<$xmlDump")) {
			binmode DUMP;
			while (<DUMP>) {
				if (@cids = /cid(?:[_-]\w+)?=\"([^"]+)\"/g) {
					foreach $fullPath (@cids) {
						unless ($fullPath =~ /^\//) {
							$fullPath = "$dirDump/$fullPath";
						}
						$fromXML{$fullPath} = undef;
					}
				}
			}
			close(DUMP);
		}

		foreach $key (keys %files) {
			unless (exists($fromXML{$key})) {
				delete $files{$key};
			}
		}
	}

  $gzip = $bin{'gzip'}||'gzip';
  $wc = $bin{'wc'}||'wc';
  $md5sum = $bin{'md5sum'}||'md5sum';

  $root=&makeXmlNode('contentlist');

  my($compressed,$uncompressed,$list,$count,$ptrList,@sortedList,
     $ptrItem,$out);
  my($totalUncomp,$totalSize)=(0,0);
  foreach $key (keys %files){

#
# sort by extension  .aa .ab ...
#
    @sortedList =  sort {&getExt($a->[0]) cmp &getExt($b->[0])} @{$files{$key}};

#
# output list of files
#
    $compressed=0;
    foreach $ptrItem (@sortedList){
      ($fullPath,$size)=@{$ptrItem};
      $item=&makeXmlNode('file',$fullPath);
      $item->{'ATTRIBUTE'}->('size',$size,'noencode');
      $root->{'ADDCHILD'}->($item);

#
# get md5sum
#
      if(-x $md5sum){
	$out=`$md5sum $fullPath`;
	if($out=~/\s*([0-9a-fA-F]+)\s+/){
	  $item->{'ATTRIBUTE'}->('md5sum',$1,'noencode');
	}
      }

      $compressed += $size;

    }

    $uncompressed = undef;
#
# get unpacked size of the archive
#
    if ($key=~/\.tgz$|\.gz$/){
      if(-x $gzip){
	if(scalar(@sortedList)==1){
	  ($fullPath,$size)=@{$sortedList[0]};
	  ($uncompressed,$compressed)=&getZipSize($fullPath,$gzip);
	}else{
	  $list = join(' ',map {$_->[0]} @sortedList);
	  if(-x $wc){
	    $cmd = "cat $list | $gzip | $wc -c";
	    $uncompressed = `$cmd`;
	    chomp $uncompressed;
	    if ($uncompressed =~ /^\s*(\d+)\s*$/){
	      $uncompressed = $1;
	    }else{
	      $uncompressed = undef;
	    }
	  }else{
	    $cmd = "cat $list >$key";
	    if(system($cmd)==0){
	      ($uncompressed,$compressed)=&getZipSize($key,$gzip);
	      unlink $key;
	    }
	  }
	}
      }
    }
#
# calc total size
#
    $totalSize += $compressed;
    $totalUncomp += $uncompressed if $uncompressed;
  }

  $root->{'ATTRIBUTE'}->('size',$totalSize,'noencode');
  if ($totalUncomp && $totalUncomp != $totalSize){
    $root->{'ATTRIBUTE'}->('unpacked',$totalUncomp,'noencode');
  }

  return $root;
}

sub getExt {
  my $path = shift;
  if($path=~/\.([^.]*)$/){
    return $1;
  }
  return $path;
}

sub getZipSize {
  my ($fullPath,$gzip) = @_;
  $gzip ||= 'gzip';
  my $cmd = "$gzip -l $fullPath";
  my ($uncompressed,$compressed,@out);
  if (open(OUTPUT,"$cmd|")){
    ($compressed,$uncompressed)=(0,0);
    while(<OUTPUT>){
      @out = split(' ',$_);
      if($out[0]=~/^\d+$/ && $out[1]=~/^\d+$/){
	$uncompressed+=$out[1];
	$compressed+=$out[0];
      }
    }
    close (OUTPUT);
  }
  return $uncompressed,$compressed;
}

my $outputIsFile;

sub openOutput
{
	my ($outPath) = @_;
	my ($outFh);

	if ($outPath) {
		open(OUTPUT, ">>$outPath") or die "Error: can not open '$outPath' for write: $!\n";
		binmode OUTPUT;
		$outFh = *OUTPUT;
		$outputIsFile = 1;
	} else {
		$outFh = *STDOUT;
		$outputIsFile = 0;
	}

	return $outFh;
}

sub closeOutput{
  if($outputIsFile){
    close (OUTPUT) or
      &printToError("Error: unable to close OUTPUT: $!\n");
    $outputIsFile=0;
  }
}

#=========================================================================
#
# dtd-specific (but common) subroutines
#
sub createPasswordNode {
  my ($parent, $password, $type) = @_;

  my $item = makeXmlNode('password', $password);
  $item->{'ATTRIBUTE'}->('type', $type);
  $parent->{'ADDCHILD'}->($item);
}

sub createSysuserNode {
  my ($parent, $name, $password, $type) = @_;

  my $item = makeXmlNode('sysuser');
  $item->{'ATTRIBUTE'}->('name', $name);
  createPasswordNode($item, $password, $type);
  $parent->{'ADDCHILD'}->($item);
}

sub readFile {
  my ($fileName) = @_;

  unless (-f $fileName) {
	return undef;
  }

  open(INPUT, "< $fileName");
  binmode INPUT;
  my $content = join('', <INPUT>);
  close(INPUT);

  return $content;
}

1;

#
# end common subroutines
#
#=========================================================================


$ptrArgs = &getArguments (\@ARGV,\%arg_opts);

#
# search binaries
#

&getBins(\%bin,'/bin:/usr/bin:/usr/local/bin:/sbin:/usr/sbin:/usr/local/sbin:'.$ENV{'PATH'});

my ($outPath,$outFh,$outHas,$dumpFile,$statusFile,$app);

$app = $0;
if ($app=~/(.+)\/([^\/]+)$/){
  $workDir = $1;
  $app = $2;
}
$app=~s/\.pl$//;

$pwd=`$bin{'pwd'}`||'.';

chop $pwd;

if($workDir eq '.'){
  $workDir = $pwd;
}elsif($workDir !~ /^\//){
  $workDir = $pwd.'/'.$workDir;
}

$dumpFile = $workDir.'/Cgp.xml';
$statusFile = $workDir.'/Cgp.status';
$dumpDir = $workDir.'/pma';
$pleskVersion = undef;
$pleskMajorVersion = undef;
$pleskMinorVersion = undef;
$pleskMinorMinorVersion = undef;

my $objDumpStatus = &makeDumpStatus($ptrArgs->{'status-file'}||$statusFile);

if(!$ptrArgs->{'output'} &&
   (exists $ptrArgs->{'dump-all'} ||
    exists $ptrArgs->{'dump-accounts'}||
    exists $ptrArgs->{'dump-domains'}
   )
  ){
  $ptrArgs->{'output'}=$dumpFile;
}

if ($outPath = $ptrArgs->{'output'}) {

	open(OUTPUT, ">$outPath") or die "Error: can not open '$outPath' for write: $!\n";
	binmode OUTPUT;
	$dumpFile = $outPath;
	close(OUTPUT);
}

if(exists $ptrArgs->{'version'}){
  print $VERSION,"\n";
  exit 0;
}

#
# get MIME Base64 encoder
#
my $wrapBase64 = makeMIMEBase64();

my (%PsaConfig,%domainsInfo);
my $rootName = 'Cgpdump';
my $dtdName = 'Cgp.dtd';

if(exists $ptrArgs->{'get-status'}){

  my $outFh = &openOutput($ptrArgs->{'output'});

  &printAgentStatus($outPath=$ptrArgs->{'output'}||$dumpFile,
		    $objDumpStatus->{'FILE'}->(),
		    $outFh);

  &closeOutput();

}elsif(exists ($ptrArgs->{'dump-all'}) ||
       exists ($ptrArgs->{'dump-accounts'}) ||
       exists ($ptrArgs->{'dump-domains'})){

  my ($root,@accounts,@domains,$ptrAccounts,$ptrDomains, $value);
  if(exists $ptrArgs->{'no-content'}){
    $::dumpDir = &getDumpDir("$workDir/pma");
  }else{
    $::dumpDir = &getDumpDir("$workDir/pma",'clean');
  }

  $version = getVersion();

  &printToLog("Work dir: $workDir");
  &printToLog("Dump file: $dumpFile");
  &printToLog("Status file: ".$objDumpStatus->{'FILE'}->());

  if ($value = $ptrArgs->{'dump-accounts'}){
	if ($value eq "-") {
	  $value = <STDIN>;
	  chomp $value;
	}
    @accounts = split(/\s*,\s*/,$value);
    $ptrAccounts = \@accounts;
  }

  if ($value = $ptrArgs->{'dump-domains'}){
	if ($value eq "-") {
	  $value = <STDIN>;
	  chomp $value;
	}
    @domains = split(/\s*,\s*/,$value);
    $ptrDomains = \@domains;
  }
#
# generate a xml dump
#

  $root = &getCgpDump($ptrAccounts, $ptrDomains, $rootName);

#
# print dump to output
#
  &printToLog('Print dump file ...',1);


  if(ref($root)=~/HASH/ && ref($root->{'PRINT'})=~/CODE/){
    my $outFh = &openOutput($ptrArgs->{'output'});
    &printXml($root,$outFh,$dtdName);
    &closeOutput();
  }


  &printToLog(' OK');

}elsif(exists $ptrArgs->{'get-content-list'}){

  $::dumpDir = &getDumpDir("$workDir/pma");

  my $root = &makeContentList($::dumpDir,$dumpFile);

  if(ref($root)=~/HASH/){
    my $outFh = &openOutput($ptrArgs->{'output'});

    &printXml($root,$outFh);

    &closeOutput();
  }

}else{
  &printHelp();
}


exit $returnCode;

#
# end main
#
#==============================================================
#
# subroutines
#

sub readAdminPassword {
  my $adminPassword;

  if (-f "migration/.shadow") {
	open PASSWD, "migration/.shadow";
	$adminPassword = <PASSWD>;
	chomp $adminPassword;
	close PASSWD;
  } elsif (-f $adminSettings) {
	open PASSWD, $adminSettings;
	while (<PASSWD>) {
	  chomp;
	  if (/\bPassword\s+=/) {
		$adminPassword = (split /=/, $_)[1];
		$adminPassword =~ s/^\s*([^;]+);$/\1/;
		chomp $adminPassword;
	  }
	}
	close PASSWD;
  }

  return $adminPassword;
}

sub readMainDomainName {
  my $mainDomainName;

  if (-f "migration/.maindomainname") {
	open MAINDOMAINNAME, "migration/.maindomainname";
	$mainDomainName = <MAINDOMAINNAME>;
	chomp $mainDomainName;
	close MAINDOMAINNAME;
  } elsif (-f $globalSettings) {
	open MAINDOMAINNAME, $globalSettings;
	while (<MAINDOMAINNAME>) {
	  chomp;
	  if (/\bDomainName\s+=/) {
		$mainDomainName = (split /=/, $_)[1];
		$mainDomainName =~ s/^\s*([^;]+);$/\1/;
		chomp $mainDomainName;
	  }
	}
	close MAINDOMAINNAME;
  }

  return $mainDomainName;
}

sub initConnectionParams {
  if ($adminLogin) {
	return;
  }

  $adminLogin = 'postmaster';
  my $password = readAdminPassword();

  my $cli = new CGP::CLI({ PeerAddr => 'localhost',
						   PeerPort => 106,
						   login => 'postmaster',
						   password => $password,
						   SecureLogin => 0 });

  if ($cli) {
	$adminPassword = $password;
  } else {
	my $host = readMainDomainName();

	$cli = new CGP::CLI({ PeerAddr => 'localhost',
						  PeerPort => 106,
						  login => 'postmaster@' . $host,
						  password => $password,
						  SecureLogin => 0 });

	if ($cli) {
	  $adminLogin = 'postmaster@' . $host;
	  $adminPassword = $password;
	}
  }
}

sub cli {

  initConnectionParams();

  if ($adminPassword) {
	return new CGP::CLI({ PeerAddr => 'localhost',
						  PeerPort => 106,
						  login => $adminLogin,
						  password => $adminPassword,
						  SecureLogin => 0
						});
  } else {
	return undef;
  }
}

sub getVersion {
  return "CommuniGate Pro";
}

sub makeCgpClient {
  my ($parent) = @_;

  my $client = makeXmlNode('client');
  $client->{'ATTRIBUTE'}->('name', 'cgp');
  $client->{'ATTRIBUTE'}->('contact', 'CommuniGate Pro Client');
  $client->{'ADDCHILD'}->(makePasswordNode('', 'plain'));

  my $ipPool = makeXmlNode('ip_pool');
  my $clientIp = makeXmlNode('client_ip');
  $clientIp->{'ATTRIBUTE'}->('ip_ref', 'main');
  $ipPool->{'ADDCHILD'}->($clientIp);
  $client->{'ADDCHILD'}->($ipPool);

  return $client;
}

sub getCgpDump {
  my ($ptrAccounts, $ptrDomains, $rootName) = @_;
  $rootName = 'Cgpdump' unless $rootName;

  my $root = makeXmlNode($rootName);

  if (ref($ptrDomains) =~ /ARRAY/) {
	dumpDomains($root, $ptrDomains);
  } else {
	dumpAll($root);
  }

  return $root;
}

sub createIpPool {
  my ($root) = @_;

  my $mapNode = makeXmlNode('map');
  my $sharedIpNode = makeXmlNode('shared');
  my $ipNode = makeXmlNode('ip');
  $ipNode->{'ATTRIBUTE'}->('id', 'main');
  my $oldNode = makeXmlNode('old', 'Server-wide IP');
  $ipNode->{'ADDCHILD'}->($oldNode);
  my $newNode = makeXmlNode('new', '');
  $ipNode->{'ADDCHILD'}->($newNode);
  $sharedIpNode->{'ADDCHILD'}->($ipNode);
  $mapNode->{'ADDCHILD'}->($sharedIpNode);
  my $exclusiveIpNode = makeXmlNode('exclusive');
  $mapNode->{'ADDCHILD'}->($exclusiveIpNode);
  $root->{'ADDCHILD'}->($mapNode);
}

sub dumpAll {
  my ($root) = @_;

  createIpPool($root);

  my $client = makeCgpClient('client');

  my $domainName;
  my $cli = cli();
  foreach $domainName (@{$cli->ListDomains()}) {
	dumpDomain($client, $domainName);
  }
  $cli->Logout();

  $root->{'ADDCHILD'}->($client);
}

sub dumpDomains {
  my ($root, $domains) = @_;

  createIpPool($root);

  my $client = makeCgpClient($root);

  my $domainName;
  foreach $domainName (@{$domains}) {
	dumpDomain($client, $domainName);
  }

  $root->{'ADDCHILD'}->($client);
}

sub dumpDomain {
  my ($clientNode, $domainName) = @_;

  printToLog("Dumping $domainName");

  my $domainNode = makeXmlNode('domain');
  $clientNode->{'ADDCHILD'}->($domainNode);

  $domainNode->{'ATTRIBUTE'}->('name', $domainName);

  my $domainIp = makeXmlNode('domain_ip');
  $domainIp->{'ATTRIBUTE'}->('ip_ref', 'main');
  $domainNode->{'ADDCHILD'}->($domainIp);

  # No more info required in 'no-content' mode
  if (exists $ptrArgs->{'no-content'}) {
	return;
  }

  my $cli = cli();
  my %settings = %{$cli->GetDomainEffectiveSettings($domainName)};
  $cli->Logout();

  if (exists $settings{'AccountsLimit'}) {
	# Due to strange limit appearing on the one of CG domains.
	if ($settings{'AccountsLimit'} < 0) {
	  $settings{'AccountsLimit'} = -1;
	}

	my $accountsLimitNode = makeXmlNode('limit', $settings{'AccountsLimit'});
	$accountsLimitNode->{'ATTRIBUTE'}->('name', 'max_box');
	$domainNode->{'ADDCHILD'}->($accountsLimitNode);
  }
  if (exists $settings{'ListsLimit'}) {
	my $listsLimitNode = makeXmlNode('limit', $settings{'ListsLimit'});
	$listsLimitNode->{'ATTRIBUTE'}->('name', 'max_maillists');
	$domainNode->{'ADDCHILD'}->($listsLimitNode);
  }

  createDomainMailsystem($domainNode, $domainName, \%settings);

#  createPhosting($cli, $domainNode, $domainName, \%settings);

}

sub createDomainMailsystem {
  my ($domainNode, $domainName, $domainSettings) = @_;

  printToLog(">Dumping mailsystem");
  my $mailsystemNode = makeXmlNode('mailsystem');

  my $cli = cli();
  my %accounts = %{$cli->ListAccounts($domainName)};
  $cli->Logout();

  my $account;
  foreach $account (keys %accounts) {
	createMailAccount($mailsystemNode, $domainName, $account);
  }

  my $cli = cli();
  my @groups = @{$cli->ListGroups($domainName)};
  $cli->Logout();

  my $group;
  foreach $group (@groups) {
	createGroup($mailsystemNode, $domainName, $group);
  }

  my $cli = cli();
  my @forwarders = @{$cli->ListForwarders($domainName)};
  $cli->Logout();

  my $forwarder;
  foreach $forwarder (@forwarders) {
	createForwarder($mailsystemNode, $domainName, $forwarder);
  }

  if (exists $domainSettings->{'MailToUnknown'} &&
	  $domainSettings->{'MailToUnknown'} eq 'Rerouted to' &&
	  $domainSettings->{'MailRerouteAddress'} ne '')
  {
	my $catchAllAddress = $domainSettings->{'MailRerouteAddress'};
	if ($catchAllAddress !~ /@/) {
	  $catchAllAddress .= '@' . $domainName;
	}
	my $catchAllNode = makeXmlNode('catch-all', $catchAllAddress);
	$mailsystemNode->{'ADDCHILD'}->($catchAllNode);
  }

  $domainNode->{'ADDCHILD'}->($mailsystemNode);
}

sub normalizeString {
  my ($str) = @_;
  $str =~ s/\\[en]/\n/g;
  $str =~ s/\\r/\r/g;
  return $str;
}

sub createMailAccount {
  my ($mailsystemNode, $domainName, $mailAccount) = @_;

  printToLog(" >Dumping $mailAccount mail account");

  my $cli = cli();
  my %settings = %{$cli->GetAccountEffectiveSettings($mailAccount . '@' . $domainName)};
  my @aliases = @{$cli->GetAccountAliases($mailAccount . '@' . $domainName)};
  $cli->Logout();

  my $mailuserNode = makeXmlNode('mailuser');
  $mailuserNode->{'ATTRIBUTE'}->('name', $mailAccount);
  $mailuserNode->{'ATTRIBUTE'}->('mailgroup-enabled', 'false');

  if ($settings{'PasswordEncryption'} ne 'clear') {
	die "Encrypted passwords are not supported yet";
  }
  $mailuserNode->{'ADDCHILD'}->(makePasswordNode($settings{'Password'}, 'plain'));

  my $multipleSessionsPermissionNode = makeXmlNode('mailuser-permission');
  $multipleSessionsPermissionNode->{'ATTRIBUTE'}->('name', 'multiple-sessions');
  $mailuserNode->{'ADDCHILD'}->($multipleSessionsPermissionNode);

  createMailaccountMailbox($mailuserNode, $domainName, $mailAccount, \%settings);

  # aliases

  my $alias;
  foreach $alias (@aliases) {
	$mailuserNode->{'ADDCHILD'}->(makeXmlNode('alias', $alias));
  }

  if (exists $settings{'Rules'}) {
	#redirect
	my $rule;

	foreach $rule (@{$settings{'Rules'}}) {
	  if ($rule->[1] eq '#Redirect') {
		my $action;
		foreach $action (@{$rule->[3]}) {
		  if($action->[0] eq 'Mirror to' || $action->[0] eq 'Redirect to') {
			my $emaillist = normalizeString($action->[1]);

			my @emails = split /[,\s]+/, $emaillist;

			if (scalar @emails > 1) {
			  if ($rule->[0] eq '0') {
				$mailuserNode->{'ATTRIBUTE'}->('mailgroup-enabled', 'false');
			  } else {
				$mailuserNode->{'ATTRIBUTE'}->('mailgroup-enabled', 'true');
			  }

			  my $email;
			  foreach $email (@emails) {
				if ($email !~ /@/) {
				  $email .= '@';
				  $email .= $domainName;
				}
				$mailuserNode->{'ADDCHILD'}->(makeXmlNode('mailgroup-member', $email));
			  }
			} elsif (scalar @emails == 1) {
			  if ($emails[0] !~ /@/) {
				$emails[0] .= '@';
				$emails[0] .= $domainName;
			  }

			  my $redirectNode = makeXmlNode('redirect', $emails[0]);

			  if ($rule->[0] eq '0') {
				$redirectNode->{'ATTRIBUTE'}->('enabled', 'false');
			  } else {
				$redirectNode->{'ATTRIBUTE'}->('enabled', 'true');
			  }

			  $mailuserNode->{'ADDCHILD'}->($redirectNode);
			}
		  }
		}
	  }
	}

	my $autorespondersNode = makeXmlNode('autoresponders');
	$autorespondersNode->{'ATTRIBUTE'}->('enabled', 'true');

	#autoresponder
	foreach $rule (@{$settings{'Rules'}}) {
	  if ($rule->[1] eq '#Vacation') { #name
		my $action;
		foreach $action (@{$rule->[3]}) {
		  if ($action->[0] eq 'Reply with') { #action type
			my $vacationMsg = normalizeString($action->[1]);

			my $autoresponderNode = makeXmlNode('autoresponder');
			$autoresponderNode->{'ATTRIBUTE'}->('name', 'Vacation');

			if ($rule->[0] eq '0') {
			  $autoresponderNode->{'ATTRIBUTE'}->('status', 'off');
			} else {
			  $autoresponderNode->{'ATTRIBUTE'}->('status', 'on');
			}

			$autoresponderNode->{'ADDCHILD'}->(makeXmlNode('text', $wrapBase64->{'ENCODE'}->($vacationMsg)));

			my $anscountLimitNode = makeXmlNode('autoresponder-limit', '1');
			$anscountLimitNode->{'ATTRIBUTE'}->('name', 'ans-count');
			$autoresponderNode->{'ADDCHILD'}->($anscountLimitNode);

			$autorespondersNode->{'ADDCHILD'}->($autoresponderNode);
		  }
		}
	  }
	}
	$mailuserNode->{'ADDCHILD'}->($autorespondersNode);
  }

  $mailsystemNode->{'ADDCHILD'}->($mailuserNode);
}

sub convertFolderNameToQmailName {
  my ($folderName) = @_;

  my @components = split /\//, $folderName;

  if (@components[0] eq "INBOX") {
	shift @components;
  }

  return "." . (join ".", @components);
}

sub convertFolderNameToCgpPath {
  my ($folderName) = @_;

  my @components = split /\//, $folderName;

  return (join ".folder/", @components) . ".mdir";
}

sub convertToQmailMdir {
  my ($toBaseDir, $fromBaseDir, $fromFolderName) = @_;

  my $from = $fromBaseDir . "/" . convertFolderNameToCgpPath($fromFolderName);
  my $to = $toBaseDir . "/" . convertFolderNameToQmailName($fromFolderName);

  unless (exists ($ptrArgs->{'no-content'})) {
	system("mkdir", "-p", "$to/cur", "$to/new", "$to/tmp", "$from");

	opendir DIR, $from;
	my @files = readdir DIR;
	closedir DIR;

	my $file;
	foreach $file (@files) {
	  unless (-f $from . "/" . $file) {
		next;
	  }
	  system("cp", "$from/$file", "$to/cur");
	}
  }
}

sub createMailaccountMailbox {
  my ($mailuserNode, $domainName, $mailAccount, $settings) = @_;

  my $cli = cli();
  my $domainLocation = $cli->GetDomainLocation($domainName);
  my $accountLocation = $cli->GetAccountLocation($mailAccount . '@' . $domainName);
  my $mboxes = $cli->ListMailboxes(accountName=> ($mailAccount . '@' . $domainName));
  $cli->Logout();

  my $mdirLocation = "/var/CommuniGate/" . $domainLocation . "/" . $accountLocation;

  my $idname = $dumpDir . "/$mailAccount" . "@" . "$domainName.mdir";
  my $srcdir = $idname . ".converted";

  my $mbox;
  foreach $mbox (keys %$mboxes) {
	printToLog("  >Dumping mailbox $mbox");
	convertToQmailMdir($srcdir, $mdirLocation, $mbox);
  }

  my $dumpId = makeDumpFile($idname, $srcdir);

  `rm -rf "$srcdir"`;

  my $mailboxNode = makeXmlNode('mailbox');
  $mailboxNode->{'ATTRIBUTE'}->('type', 'mdir');
  $mailboxNode->{'ATTRIBUTE'}->('cid', $dumpId);

  if (exists $settings->{'MaxAccountSize'}) {
	my $quota = $settings->{'MaxAccountSize'};
	if ($quota =~ /^(\d+)K$/) {
	  $quota = $1 * $kilobyte;
	} elsif ($quota =~ /^(\d+)M$/) {
	  $quota = $1 * $megabyte;
	} elsif ($quota =~ /^(\d+)G$/) {
	  $quota = $1 * $gigabyte;
	} elsif ($quota =~ /unlimited/) {
	  $quota = -1;
	}
	$mailuserNode->{'ATTRIBUTE'}->('mailbox-quota', $quota);
  }

  $mailuserNode->{'ADDCHILD'}->($mailboxNode);
}

sub createGroup {
  my ($mailsystemNode, $domainName, $group) = @_;

  printToLog('  >Dumping mailgroup ' . $group);

  my $cli = cli();
  my $settings = $cli->GetGroup($group . '@' . $domainName);
  $cli->Logout();

  my $mailuserNode = makeXmlNode('mailuser');
  $mailuserNode->{'ATTRIBUTE'}->('name', $group);
  $mailuserNode->{'ATTRIBUTE'}->('mailgroup-enabled', 'true');

  my %members;
  my $member;
  foreach $member (@{$settings->{'Members'}}) {
	if ($member =~ /#(.*)$/) { #skip delivery to mailboxes
	  $member = $1;
	}

	$members{$member} = 1;
  }

  foreach $member (keys %members) {
	$mailuserNode->{'ADDCHILD'}->(makeXmlNode('mailgroup-member', $member));
  }

  $mailsystemNode->{'ADDCHILD'}->($mailuserNode);
}

sub createForwarder {
  my ($mailsystemNode, $domainName, $forwarderName) = @_;

  printToLog('  >Dumping forwarder ' . $forwarderName);

  my $cli = cli();
  my $forwardMail = $cli->GetForwarder($forwarderName . '@' . $domainName);
  $cli->Logout();

  if ($forwardMail !~ '@') {
	$forwardMail .= '@' . $domainName;
  }

  my $mailuserNode = makeXmlNode('mailuser');
  $mailuserNode->{'ATTRIBUTE'}->('name', $forwarderName);
  $mailuserNode->{'ATTRIBUTE'}->('mailgroup-enabled', 'false');
  $mailuserNode->{'ADDCHILD'}->(makeXmlNode('redirect', $forwardMail));

  $mailsystemNode->{'ADDCHILD'}->($mailuserNode);
}

sub makeEmptyDump {
  my($rootName)=@_;
  my $root=&makeXmlNode($rootName);

  my $mapRoot = &makeXmlNode('map');
  $root->{'ADDCHILD'}->($mapRoot);

  my $item = &makeXmlNode('shared');
  $mapRoot->{'ADDCHILD'}->($item);

  $item = &makeXmlNode('exclusive');
  $mapRoot->{'ADDCHILD'}->($item);

  return $root;
}

sub printAgentStatus {
  my $dumpFile = shift;
  my $statusFile = shift;
  my $outFh = shift || *STDOUT;
  my $rootName = shift || 'agentstatus';

  my ($root,$util,$out,$item,$engineRoot,$version);

  $root = &makeXmlNode($rootName);

  my $cli = cli();

  unless ((defined $bin{'iconv'}) && (defined $cli)) {
	unless ($cli) {
	  $item = makeXmlNode('wrong-platform', 'Unable to determine CommuniGate Pro' .
' postmaster\'s password. Probably this host does not contain CommmuniGate Pro' .
' installation or uses encrypted passwords. If host uses encrypted passwords,' .
' please provide plaintext password of postmaster in the file ' .
' migration/.shadow in the homedir of user you are choosing at the first' .
' page of migration manager.');
	} elsif (not defined $bin{'iconv'}) {
	  $item = makeXmlNode('wrong-platform', 'no iconv found on the source host');
	} else {
	  $item = makeXmlNode('wrong-platform');
	}

    $root->{'ADDCHILD'}->($item);
    &printXml($root,$outFh,$rootName.'.dtd');
    return;
  }

  $item = &makeXmlNode('dumpfile',$dumpFile);
  $root->{'ADDCHILD'}->($item);
  $item = &makeXmlNode('statusfile',$statusFile);
  $root->{'ADDCHILD'}->($item);

  &printXml($root,$outFh,$rootName.'.dtd');
}


sub printHelp {

  print <<"HELP";

Agent of migration from 'CommuniGate Pro' to 'Plesk 7'

Usage:
  $0 <options>

Options:
  -c |--config=<path>        path to 'psa.conf'

  -s |--get-status           get status of the agent
  -dc|--dump-accounts=<list> a coma separated list of resellers to dump
  -dd|--dump-domains=<list>  a coma separated list of domains   to dump
  -da|--dump-all             make a full dump

  -lc|--get-content-list     get list of content files
  -nc|--no-content           do not make content files

  -o |--output=<path>        redirect output to file

  -h |--help                 this help
  -d |--debug=<level>        level of debug messages
  -v |--version

HELP
}

1;
