<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <!--
       @desc Ensim 3.5.x, 4.0
  -->

  <xsl:output indent="yes" encoding="UTF-8"/>

  <xsl:template match='EnsimXdump'>
    <xsl:text disable-output-escaping="yes">&lt;!DOCTYPE plesk7dump SYSTEM 'Plesk70.dtd'&gt;</xsl:text>
    <plesk7dump>
      <xsl:call-template name="create-ip-map"/>

      <xsl:for-each select="reseller">
        <xsl:call-template name="create-client"/>
      </xsl:for-each>
    </plesk7dump>
  </xsl:template>

  <!-- Plesk does not allow to create users listed in xsl:when construction below, so we have to mangle it -->

  <xsl:template name="mangle-very-sys-user">
    <xsl:param name="username"/>
    <xsl:choose>
      <xsl:when test="$username = 'admin' or $username = 'ftp' or $username = 'anonymous' or $username = 'root' or $username = 'mailman' or $username = 'tomcat4'"><xsl:value-of select="concat($username, '1')"/></xsl:when>
      <xsl:otherwise><xsl:value-of select="$username"/></xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:key name="ip-shared"
    match="domain[hosting-type/@type = 'name-based']"
    use="domain-ip"/>

  <xsl:template name="create-ip-map">
    <map>
      <shared>
        <xsl:for-each select="//domain[generate-id(.) = generate-id(key('ip-shared', domain-ip))]">
          <ip id="{generate-id(.)}">
            <old><xsl:value-of select="domain-ip"/></old>
            <new><xsl:value-of select="domain-ip"/></new>
          </ip>
        </xsl:for-each>
      </shared>
      <exclusive>
        <xsl:for-each select="//domain[hosting-type/@type != 'name-based']">
          <ip id="{generate-id(.)}">
            <old><xsl:value-of select="domain-ip"/></old>
            <new><xsl:value-of select="domain-ip"/></new>
          </ip>
        </xsl:for-each>
      </exclusive>
    </map>
  </xsl:template>

  <xsl:template name="create-client">
    <client name="{pinfo/@username}" contact="{pinfo/@fullname}">

      <!-- password -->
      <xsl:copy-of select="password"/>

      <!-- status? -->
      <xsl:choose>
        <xsl:when test="disabled">
          <status>false</status>
        </xsl:when>
        <xsl:otherwise>
          <status>true</status>
        </xsl:otherwise>
      </xsl:choose>

      <!-- pinfo* -->

      <!-- limit* -->
      <xsl:variable name="disk-quota" select="reseller-limit[@name = 'disk-quota']"/>
      <xsl:if test="$disk-quota">
        <limit name="disk_space"><xsl:value-of select="$disk-quota"/></limit>
      </xsl:if>

      <xsl:variable name="traffic-quota" select="reseller-limit[@name = 'traffic-quota']"/>
      <xsl:if test="$traffic-quota">
        <limit name="max_traffic"><xsl:value-of select="$traffic-quota"/></limit>
      </xsl:if>

      <xsl:variable name="max-users" select="reseller-limit[@name = 'max-users']"/>
      <xsl:if test="$max-users">
        <limit name="max_wu"><xsl:value-of select="$max-users"/></limit>
      </xsl:if>

      <xsl:variable name="ip-based-domains" select="reseller-limit[@name = 'ip-based-domains']"/>
      <xsl:variable name="name-based-domains" select="reseller-limit[@name = 'name-based-domains']"/>
      <xsl:if test="$ip-based-domains or $name-based-domains">
        <xsl:choose>
          <xsl:when test="$ip-based-domains = -1 or $name-based-domains = -1">
            <limit name="max_dom">-1</limit>
          </xsl:when>
          <xsl:otherwise>
            <limit name="max_dom"><xsl:value-of select="$ip-based-domains + $name-based-domains"/></limit>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>

      <!-- permission* -->

      <xsl:if test="reseller-permission[@name='create-ipbased'] or reseller-permission[@name='create-namebased']">
        <permission name="create_domains"/>
      </xsl:if>

      <!-- FIXME: create-users -->

      <!-- ip_pool? -->
      <ip_pool>
        <xsl:for-each select="domain">
          <xsl:choose>
            <xsl:when test="hosting-type[@type = 'name-based']">
              <client_ip ip_ref="{generate-id(//domain[generate-id(.)=generate-id(key('ip-shared', current()/domain-ip))])}"/>
            </xsl:when>
            <xsl:otherwise>
              <client_ip ip_ref="{generate-id(.)}"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:for-each>
      </ip_pool>

      <!-- domain* -->

      <xsl:for-each select="domain">
        <xsl:call-template name="create-domain"/>
      </xsl:for-each>
    </client>
  </xsl:template>

  <xsl:template name="create-domain">
    <domain name="{@name}">

      <!-- domain_ip -->
      <xsl:choose>
        <xsl:when test="hosting-type[@type = 'name-based']">
          <domain_ip ip_ref="{generate-id(//domain[generate-id(.)=generate-id(key('ip-shared', current()/domain-ip))])}"/>
        </xsl:when>
        <xsl:otherwise>
          <domain_ip ip_ref="{generate-id(.)}"/>
        </xsl:otherwise>
      </xsl:choose>

      <!-- limit* -->
      <xsl:variable name="max-subdomain" select="domain-limit[@name = 'max-subdomain']"/>
      <xsl:if test="$max-subdomain">
        <limit name="max_subdom"><xsl:value-of select="$max-subdomain"/></limit>
      </xsl:if>

      <xsl:variable name="users" select="domain-limit[@name = 'users']"/>
      <xsl:if test="$users">
        <limit name="max_wu"><xsl:value-of select="$users"/></limit>
      </xsl:if>

      <xsl:variable name="disk-quota" select="domain-limit[@name = 'disk-quota']"/>
      <xsl:if test="$disk-quota">
        <limit name="disk_space"><xsl:value-of select="$disk-quota"/></limit>
      </xsl:if>

      <xsl:variable name="traffic-quota" select="domain-limit[@name = 'traffic-quota']"/>
      <xsl:if test="$traffic-quota">
        <limit name="max_traffic"><xsl:value-of select="$traffic-quota"/></limit>
      </xsl:if>

      <!-- permission* -->
      <!-- FIXME: E-COMMERCE and DEVTOOL are currently IGNORED! -->

      <xsl:if test="domain-permission[@name='create-subdomains']">
        <permission name="manage_subdomains"/>
      </xsl:if>

      <!-- FIXME: IS FILE MANAGER ALWAYS ACCESSIBLE??? -->
      <xsl:if test="domain-permission[@name='file-manager']">
      </xsl:if>

      <!-- backup moved to domuser -->
      <!-- telnet/ssh moved to phosting -->

      <!-- mailsystem? -->
      <xsl:if test="email">
        <mailsystem>
          <xsl:copy-of select="email/mailuser"/>
		  <catch-all><xsl:value-of select="domain-admin/pinfo/@username"/>@<xsl:value-of select="@name"/></catch-all>
        </mailsystem>
      </xsl:if>

      <!-- dnsrec* -->

<!--
	 <dns enabled="true">
	  <xsl:variable name="hostname" select="@name"/>
      <xsl:for-each select="domain-alias">
        <dnsrec type="CNAME" src="{text()}" dst="{$hostname}"/>
      </xsl:for-each>

      <xsl:if test="web">
        <dnsrec type="CNAME" src="{web/@web-server-name}" dst="{$hostname}"/>
      </xsl:if>

      <xsl:if test="email">
        <dnsrec type="A" src="{email/@mail-server-name}" dst="{domain-ip/text()}"/>
      </xsl:if>

      <xsl:if test="ftp">
        <dnsrec type="CNAME" src="{ftp/@ftp-server-name}" dst="{$hostname}"/>
      </xsl:if>
     </dns>
-->
      <!-- database* -->
	  <xsl:copy-of select="database"/>

      <!-- maillist* -->
	  <xsl:copy-of select="maillist"/>

      <!-- traffic? --> <!-- FIXME: work with it -->

      <!-- status? --> <!-- FIXME: now always enabled -->
      <status>true</status>

      <!-- certificate* --> <!-- FIXME: no certificates migration -->

      <!-- domainuser -->
      <domainuser contact="{domain-admin/pinfo/@username}" status="true">
        <xsl:copy-of select="domain-admin/password"/>
      </domainuser>

      <!-- (shosting|fhosting|phosting)? --> <!-- FIXME: make it works -->

      <phosting>

        <xsl:if test="web/web-permission[@name='frontpage']">
          <xsl:attribute name="fp">true</xsl:attribute>
        </xsl:if>

        <xsl:if test="monitoring/monitoring-permission[@name='analog' or @name='webalizer']">
          <xsl:attribute name="webstat">true</xsl:attribute>
        </xsl:if>

        <xsl:copy-of select="@cid_docroot"/>
        <xsl:copy-of select="@cid_cgi"/>

		<xsl:if test="domain-permission[@name='ssl']">
		  <xsl:attribute name="https">true</xsl:attribute>
		</xsl:if>

		<xsl:if test="domain-permission[@name='ssl'] and domain-permision[@name='frontpage ']">
		  <xsl:attribute name="fpssl">true</xsl:attribute>
		</xsl:if>

        <xsl:variable name="domain" select="."/>

        <xsl:variable name="sysusername">
          <xsl:for-each select="//domain[domain-admin/pinfo/@username = $domain/domain-admin/pinfo/@username]">
            <xsl:if test="@name = $domain/@name">
              <xsl:variable name="admin-name" select="$domain/domain-admin/pinfo/@username"/>
              <xsl:choose>
                <xsl:when test="position() = 1">
                  <xsl:call-template name="mangle-very-sys-user">
                    <xsl:with-param name="username" select="$admin-name"/>
                  </xsl:call-template>
                </xsl:when>
                <xsl:otherwise><xsl:value-of select="concat($admin-name, position())"/></xsl:otherwise>
              </xsl:choose>
            </xsl:if>
          </xsl:for-each>
        </xsl:variable>

        <sysuser name='{$sysusername}'>
          <xsl:if test="domain-permission[@name='ssh' or @name='telnet']">
            <xsl:attribute name="shell">/bin/bash</xsl:attribute>
          </xsl:if>
          <xsl:copy-of select="domain-admin/password"/>
        </sysuser>

        <!-- scripting -->

		  <!-- php is unconditionally enabled on Ensim -->
        <scripting php="true">
          <xsl:if test="web/web-permission[@name='cgi']">
            <xsl:attribute name='cgi'>true</xsl:attribute>
          </xsl:if>
          <xsl:if test="web/web-permission[@name='mod_perl']">
            <xsl:attribute name='perl'>true</xsl:attribute>
          </xsl:if>
          <xsl:if test="web/web-permission[@name='ssi']">
            <xsl:attribute name='ssi'>true</xsl:attribute>
          </xsl:if>
        </scripting>

        <!-- fpuser -->

        <xsl:if test="web/web-permission[@name='frontpage']">
          <fpuser name="{$sysusername}">
            <xsl:copy-of select="domain-admin/password"/>
          </fpuser>
        </xsl:if>

        <!-- logrotation -->
        <xsl:copy-of select="logrotation"/>

        <!-- anonftp -->

        <xsl:if test="ftp/ftp-permission[@name='anon-ftp']">
          <anonftp pub="true" incoming="true" display-login="false">
            <xsl:copy-of select="ftp/@cid"/>
            <xsl:copy-of select="ftp/@cid_incoming"/>

            <xsl:if test="domain-limit[@name = 'disk-quota']">
              <anonftp-limit name="incoming-disk-quota"><xsl:value-of select="domain-limit[@name='disk-quota']/text()"/></anonftp-limit>
            </xsl:if>

            <anonftp-permission name="incoming-mkdir"/>
            <anonftp-permission name="incoming-download"/>
          </anonftp>
        </xsl:if>

        <!-- pdir -->
        <xsl:copy-of select="pdir"/>

        <!-- webuser -->
        <xsl:for-each select="domain-user[@cid]">
          <xsl:variable name="user" select="."/>
          <xsl:variable name="username" select="pinfo/@username"/>
          <xsl:for-each select="//domain[domain-user/pinfo/@username = $username]">
            <xsl:if test="@name = $domain/@name">
              <webuser>
                <xsl:copy-of select="$user/@cid"/>

                <sysuser>
				  <xsl:attribute name="name">
					<xsl:call-template name="mangle-very-sys-user">
					  <xsl:with-param name="username">
						<xsl:choose>
						  <xsl:when test="position() = 1">
							<xsl:value-of select="$username"/>
						  </xsl:when>
						  <xsl:otherwise>
							<xsl:value-of select="concat($username, position())"/>
						  </xsl:otherwise>
						</xsl:choose>
					  </xsl:with-param>
					</xsl:call-template>
				  </xsl:attribute>
                  <xsl:copy-of select="$user/password"/>

				  <xsl:if test="domain-user-limit[@name = 'disk-quota']">
					<xsl:attribute name="quota">
					  <xsl:value-of select="domain-user-limit[@name = 'disk-quota']/text()"/>
					</xsl:attribute>
				  </xsl:if>
                </sysuser>

                <scripting ssi="false" php="true" perl="false" asp="false" python="false" coldfusion="false">
                  <xsl:choose>
                    <xsl:when test="$user/domain-user-permission[@name='cgi']">
                      <xsl:attribute name="cgi">true</xsl:attribute>
                    </xsl:when>
                    <xsl:otherwise>
                      <xsl:attribute name="cgi">false</xsl:attribute>
                    </xsl:otherwise>
                  </xsl:choose>
                </scripting>
              </webuser>
            </xsl:if>
          </xsl:for-each>
        </xsl:for-each>

        <!-- subdomain -->

        <xsl:for-each select="subdomain">
          <subdomain name="{@name}">
            <xsl:copy-of select="@cid_docroot"/>
            <xsl:copy-of select="@cid_cgi"/>
            <scripting ssi="false" php="true" perl="false" asp="false" python="false" coldfusion="false">
              <xsl:choose>
                <xsl:when test="subdomain-permission/@name = 'cgi'">
                  <xsl:attribute name="cgi">true</xsl:attribute>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:attribute name="cgi">false</xsl:attribute>
                </xsl:otherwise>
              </xsl:choose>
            </scripting>
          </subdomain>
        </xsl:for-each>
      </phosting>

    </domain>
  </xsl:template>
</xsl:stylesheet>
